#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "buchberger.h"
#include "wallideal.h"
#include "lp.h"
#include "reversesearch.h"
#include "termorder.h"
#include "ep_standard.h"
#include "ep_xfig.h"
#include "gfanapplication.h"
#include "symmetry.h"
#include "halfopencone.h"
#include "newtonpolytope.h"
#include "subspace.h"

class BerndsSuggestionApplication : public GFanApplication
{
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program computes.\n";
  }
  BerndsSuggestionApplication()
  {
    registerOptions();
  }
  char *name()
  {
    return "_berndssuggestion";
  }
  vector<set<Subspace> > toFan(PolynomialSet const &g)
  {
    int n=g.getRing().getNumberOfVariables();
    vector<set<Subspace> > ret;

    for(PolynomialSet::const_iterator i=g.begin();i!=g.end();i++)
      {
	IntegerVectorList L=newtonPolytope(*i);

	set<Subspace> a;
	for(IntegerVectorList::const_iterator j=L.begin();j!=L.end();j++)
	  for(IntegerVectorList::const_iterator k=L.begin();k!=j;k++)
	    {
	      IntegerVectorList empty;
	      IntegerVectorList equations;
	      equations.push_back(*k-*j);
	      a.insert(Subspace(equations,n));
	    }
	ret.push_back(a);
      }

    return ret;
  }
  /*  IntegerVector fVector(set<Subspace> const &s, int n)
  {
    IntegerVector ret(n+1);
    for(set<Subspace>::const_iterator i=s.begin();i!=s.end();i++)
      {
	ret[
      }
    retun ret;
    }*/
  int main()
  {
    FileParser P(Stdin);

    PolynomialSet g=P.parsePolynomialSetWithRing();

    int n=g.getRing().getNumberOfVariables();

    IntegerVectorList empty;

    vector<set<Subspace> > fans=toFan(g);

    set<Subspace> A;
    A.insert(Subspace(empty,n));

    for(int i=0;i<fans.size();i++)
      {
	set<Subspace> B;
	for(set<Subspace>::const_iterator j=A.begin();j!=A.end();j++)
	  for(set<Subspace>::const_iterator k=fans[i].begin();k!=fans[i].end();k++)
	    {
	      Subspace temp=sum(*j,*k);
	      //temp.canonicalize();
	      B.insert(temp);
	    }
	fprintf(Stderr,"Size:%i\n",B.size());
	A=B;
      }

    return 0;
  }
};

static BerndsSuggestionApplication theApplication;
