#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "lp.h"
#include "gfanapplication.h"
#include "polyhedralcone.h"

#include "polymakefile.h"
#include "determinant.h"
#include "subspace.h"
#include "triangulation.h"

#include "symmetry.h"

class CombineRaysApplication : public GFanApplication
{
  StringOption inputOption;
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program combines rays from the specified polymake file by adding according to a list of vectors of indices given on the standard input.\n";
  }
  CombineRaysApplication():
    inputOption("-i","Specify the name of the input file.","examples/grassmann3_7.out")
    //    outputOption("-o","Specify the name of the output file.","polymake.out2")
  {
    registerOptions();
  }    

  char *name()
  {
    return "_combinerays";
  }

  int main()
  {
    PolymakeFile inFile;
    inFile.open(inputOption.getValue());

    int N=inFile.readCardinalProperty("AMBIENT_DIM");

    int nRays=inFile.readCardinalProperty("N_RAYS");
    IntegerMatrix rays=inFile.readMatrixProperty("RAYS",nRays,N);
    

    FileParser P(Stdin);

    IntegerVectorList comb=P.parseIntegerVectorList();

    IntegerVectorList result;
    for(IntegerVectorList::const_iterator j=comb.begin();j!=comb.end();j++)
      {
	IntegerVector interiorPoint(N);
	for(int i=0;i<j->size();i++)
	  {
	    interiorPoint+=rays[(*j)[i]];
	  }
	result.push_back(interiorPoint);
      }

    AsciiPrinter(Stdout).printVectorList(result);
    fprintf(Stdout,"\n");

    return 0;
  }
};

static CombineRaysApplication theApplication;
