/* This application is just for the project with Sven, Bernd and Michael */

#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "lp.h"
#include "gfanapplication.h"
#include "polyhedralcone.h"

#include "polymakefile.h"
#include "determinant.h"
#include "subspace.h"
#include "triangulation.h"

#include "symmetry.h"
#include "regularsubdivision.h"
#include "symmetriccomplex.h"

#include "buchberger.h"
#include "polyhedralfan.h"
#include "wallideal.h"
#include "tropical2.h"

#include "field_zmodpz.h"

#include "tropical.h"

class GrassmannDataApplication : public GFanApplication
{
  //  StringOption inputOption;
  //  StringOption outputOption;
  IntegerOption dOption;
  IntegerOption nOption;
  IntegerOption cOption;
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program ........\n";
  }
  GrassmannDataApplication():
    cOption("-c","Perform modulo test. Value is the characteristic of the field to be checked. O means no such check.",0),
    dOption("-d","Specify d.",3),
    nOption("-n","Specify n.",7)
    //    inputOption("-i","Specify the name of the input file.","examples/grassmann3_7.conesrep")
    //    outputOption("-o","Specify the name of the output file.","polymake.out2")
  {
    registerOptions();
  }    

  char *name()
  {
    return "_grassmanndata";
  }

  void printIntList(list<int> const &v)
  {
    FILE *f=Stderr;
    fprintf(f,"{");
    for(list<int>::const_iterator i=v.begin();i!=v.end();i++)
      {
	if(i!=v.begin())fprintf(f," ");
	fprintf(f,"%i",*i);
      }
    fprintf(f,"}\n");
  }
  void printIntListList(list<list<int> > const &l)
  {
    for(list<list<int> >::const_iterator i=l.begin();i!=l.end();i++)
      printIntList(*i);
  }

  int numberOfIntegers(set<set<int > > const &div)
  {
    int ret=0;
    for(set<set<int > >::const_iterator i=div.begin();i!=div.end();i++)
      {
	ret+=i->size();
      }
    return ret;
  }
  set<set<int> > permutedSubdivision(set<set<int> > const &div, IntegerVector const &perm)
  {
    set<set<int > > ret;

    for(set<set<int> >::const_iterator i=div.begin();i!=div.end();i++)
      {
	set<int> temp;
	for(set<int>::const_iterator j=i->begin();j!=i->end();j++)
	  {
	    temp.insert(perm[*j]);
	  }

	ret.insert(temp);
      }

    return ret;
  }
  int subdivisionOrbit(list<set<set<int > > > &subdivisions, SymmetryGroup const &s, set<set<int> > const &div, IntegerVector &perm)
  {
    int I=0;
    for(list<set<set<int > > >::const_iterator i=subdivisions.begin();i!=subdivisions.end();i++,I++)
      {
	if(numberOfIntegers(*i)==numberOfIntegers(div))
	  for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
	    {
	      if(permutedSubdivision(div,*j)==*i)
		{
		  perm=*j;
		  return I;
		}
	    }
      }
    subdivisions.push_back(div);
    perm=SymmetryGroup::identity(s.sizeOfBaseSet());
    return I;
  }

  void printBasis(int j, FILE *f, int d)
  {
	    fprintf(f,"<basis>\n");
	    
	    //    int d=7;
    int choose=3;
    vector<int> I;
    for(int i=0;i<choose;i++)I.push_back(1);   // New ordering,works for n=6
    for(int i=0;i<d-choose;i++)I.push_back(0); //
    //    for(int i=0;i<d-choose;i++)I.push_back(0);
    //    for(int i=0;i<choose;i++)I.push_back(1);

    for(int i=0;i<j;i++)
      prev_permutation(I.begin(),I.end());     // New ordering
    //      next_permutation(I.begin(),I.end());

    for(int i=0;i<I.size();i++)
      if(I[i])fprintf(f,"%i ",i);
	    fprintf(f,"</basis>\n");
  }

  void printMatroidSubdivision(set<set<int> > const &mList, int n)
  {
    FILE *f=stdout;
    fprintf(f,"<matroidsubdivision>");
    for(set<set<int> >::const_iterator i=mList.begin();i!=mList.end();i++)
      {
	fprintf(f,"<matroid>\n");
	for(set<int>::const_iterator j=i->begin();j!=i->end();j++)
	  {
	    printBasis(*j,f,n);
	  }
	fprintf(f,"</matroid>\n");
      }
    fprintf(f,"</matroidsubdivision>\n");
  }

  PolyhedralCone groebnerCone(PolynomialSet g, IntegerVector const &w)
  {
    WeightReverseLexicographicTermOrder T(w);
    buchberger(&g,T);

    IntegerVectorList equalities=wallInequalities(initialForms(g,w));
    IntegerVectorList normals=algebraicTest(wallInequalities(g),g);

    PolyhedralCone c(normals,equalities,g.getRing().getNumberOfVariables());
    c.canonicalize();

    return c;
  }


  IntegerVectorList computeInteriorPoints(IntegerMatrix const &rays, vector<list<int> > const &cones)
  {
    IntegerVectorList ret;
    for(vector<list<int> >::const_iterator i=cones.begin();i!=cones.end();i++)
      {
	IntegerVector interiorPoint(rays.getWidth());
	for(list<int>::const_iterator j=i->begin();j!=i->end();j++)
	  {
	    interiorPoint+=rays[*j];
	  }
	ret.push_back(interiorPoint);
      }
    return ret;
  }

  void moduloTest(int p, IntegerVectorList pointsToCheck)
  {
    FieldZModPZ f(p);

    PolynomialRing R(f,subsetVariableNames("p",7,3,true));//USING M2 CONVENTION

    FILE *f1=fopen("examples/grassmann3_7.withsym","r");
    assert(f1);
    PolynomialRing R2=FileParser(f1).parsePolynomialRing();
    PolynomialSet g=FileParser(f1).parsePolynomialSet(R);

    int I=0;
    for(IntegerVectorList::const_iterator i=pointsToCheck.begin();i!=pointsToCheck.end();i++,I++)
      {
	bool c=containsMonomial(initialIdeal(g,*i));
	fprintf(Stderr,"%i,%i\n",I,c);
      }
  }

  char *filenameConesRep(int n)
  {
    if(n==6)
      return "grassmann3_6.conesrep";
       
    if(n==7)
      return "examples/grassmann3_7.conesrep";
    return "UNKOWNFILE";
  }
  char *filenameWithSym(int n)
  {
    if(n==6)
      return "grassmann3_6NEWCONVENTION";
       
    if(n==7)
      return "examples/grassmann3_7.withsym";
    return "UNKOWNFILE";
  }
  char *filenameBVectors(int n)
  {
    if(n==6)
      return "grassmann3_6.bvectors";
       
    if(n==7)
      return "examples/G3_7/bvectors";
    return "UNKOWNFILE";
  }
  char *filenameHyperSimplex(int n)
  {
    if(n==6)
      return "hypersimplex3_6";
       
    if(n==7)
      return "examples/G3_7/hypersimplex";
    return "UNKOWNFILE";
  }
  int nChoose3(int n)
  {
    if(n==6)return 20;
    if(n==7)return 35;
    assert(0);
    return 0;
  }

    IntegerVector representative(IntegerVector const &v, SymmetryGroup const &s)
    {
      IntegerVector ret=v;
      for(SymmetryGroup::ElementContainer::const_iterator k=s.elements.begin();k!=s.elements.end();k++)
	{
	  if(SymmetryGroup::compose(*k,v)<ret)ret=SymmetryGroup::compose(*k,v);
	}
      return ret;
    }

  char *htmlColor(int i)
  {
    static char s[20];
    int j=((31-i*16)&31)+224;
    sprintf(s,"%2X%2X%2X",j,j,j);
    return s;
  }
  list<int> remap(list<int> const&s, IntegerMatrix const &m, IntegerVector const &perm)
  {
    set<int> ret2;
    for(list<int>::const_iterator i=s.begin();i!=s.end();i++)
      {
	IntegerVector v=SymmetryGroup::composeInverse(perm,m[*i]); //INVERTED
	bool found=false;
	for(int j=0;j<m.getHeight();j++)
	  if(v==m[j])
	    {
	      found=true;
	      ret2.insert(j);
	      break;
	    }
	assert(found);
      }
    list<int> ret;
    for(set<int>::const_iterator i=ret2.begin();i!=ret2.end();i++)ret.push_back(*i);
    return ret;
  }
  int subdivisionOrbitSize(set<set<int> > const &div, SymmetryGroup const &s)
  {
    int a=0;
    for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
      if(permutedSubdivision(div,*j)==div)a++;
    return s.elements.size()/a;
  }
  int main()
  {
    int d=dOption.getValue();
    int n=nOption.getValue();

    PolymakeFile inFile;
    fprintf(Stderr,"Test\n");
    inFile.open(filenameConesRep(n));
    fprintf(Stderr,"Test\n");


    IntegerVector finalPermutation=(n==6)?SymmetryGroup::identity(20):StringParser("(34,33,32,31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0)").parseIntegerVector();

    int N=inFile.readCardinalProperty("AMBIENT_DIM");
    //int d=inFile.readCardinalProperty("DIM");
    //int nInteriorPoints=inFile.readCardinalProperty("N_INTERIOR_POINTS");

    int nRays=inFile.readCardinalProperty("N_RAYS");
    IntegerMatrix rays=inFile.readMatrixProperty("RAYS",nRays,N);



    vector<list<int> > cones=inFile.readMatrixIncidenceProperty("MAXIMAL_CONES_COMPRESSED");

    if(cOption.getValue())
    {
      moduloTest(cOption.getValue(),computeInteriorPoints(rays,cones));
      return 0;
    }


    FILE *f1=fopen(filenameWithSym(n),"r");
    assert(f1);
    PolynomialSet g=FileParser(f1).parsePolynomialSetWithRing();

    SymmetryGroup s(g.numberOfVariablesInRing());
    IntegerVectorList generators=FileParser(f1).parseIntegerVectorList();
	s.computeClosure(generators);


    fclose(f1);


    IntegerMatrix rayColors(rays.getHeight(),1);
    vector<IntegerVector> raysReps;
    {
      for(int i=0;i<rays.getHeight();i++)
	{
	  bool found=false;
	  IntegerVector rep=representative(rays[i],s);	  
	  for(int j=0;j<raysReps.size();j++)
	    if(raysReps[j]==rep)found=true;
	  if(!found)
	    raysReps.push_back(rep);
	}
      for(int i=0;i<rays.getHeight();i++)
	{
	  IntegerVector rep=representative(rays[i],s);
	  int J=0;
	  for(vector<IntegerVector>::const_iterator j=raysReps.begin();j!=raysReps.end();j++,J++)
	    if(*j==rep){rayColors[i][0]=J;break;}
	}
    }
    fprintf(Stderr,"sizeof elementscontainer:%i , sizeofraysrep %i\n",s.elements.size(),raysReps.size());
    for(vector<IntegerVector>::const_iterator i=raysReps.begin();i!=raysReps.end();i++)
      AsciiPrinter(Stderr).printVector(*i);

    FILE *f3=fopen(filenameBVectors(n),"r");
    assert(f3);
    vector<string> bvectors(125);
    for(int i=0;i<125;i++)
      {
	char line[4096];
	fgets(line,4095,f3);
	bvectors[i]=string(line);
      }
    fclose(f3);


    FILE *f2=fopen(filenameHyperSimplex(n),"r");
    assert(f2);
    IntegerMatrix hyperSimplex=rowsToIntegerMatrix(FileParser(f2).parseIntegerVectorList());
    fclose(f2);


    //    nInteriorPoints=2;

    //    fprintf(Stderr,"%i %i\n",n,nRays);

    //    IntegerMatrix interiorPoints=inFile.readMatrixProperty("INTERIOR_POINTS",nInteriorPoints,N);


    SymmetricComplex complex(nChoose3(n),rays.getRows(),s);
      for(int i=0;i<cones.size();i++)
	{
	  set<int> temp;
	  IntegerVectorList normals;
	  for(list<int>::const_iterator j=cones[i].begin();j!=cones[i].end();j++)
	    {
	      temp.insert(*j);
	    }
	  //	  complex.insert(SymmetricComplex::Cone(temp,13,1,true,complex));
	}


    fprintf(stdout,"<orbits>\n");
    fprintf(stdout,"<n>%i</n>\n",n);

    list<set<set<int> > > subdivisions;

    vector<int> tempCones(cones.size());

    for(int i=0;i<cones.size();i++)
      {
	fprintf(Stderr,"%i\n",i);
	//if(i==10)break;


	IntegerVector interiorPoint(N);
	for(list<int>::const_iterator j=cones[i].begin();j!=cones[i].end();j++)
	  {
	    interiorPoint+=rays[*j];
	  }

	IntegerVector perm;
	int subdivision=subdivisionOrbit(subdivisions,s,regularSubdivision(hyperSimplex,interiorPoint),perm);
	cones[i]=remap(cones[i],rays,perm);
	tempCones[i]=subdivision;
      }

    list<set<set<int> > >::const_iterator I=subdivisions.begin();
    for(int i=0;i<subdivisions.size();i++,I++)
      {
	fprintf(Stderr,"%i\n",i);
	fprintf(stdout,"<subdivision>\n");
	fprintf(stdout,"<subdivisionIndex>\n");
	fprintf(Stdout,"%i",i);
	fprintf(stdout,"</subdivisionIndex>\n");
	fprintf(stdout,"<subdivisionOrbitSize>\n");
	fprintf(Stdout,"%i",subdivisionOrbitSize(*I,s));
	fprintf(stdout,"</subdivisionOrbitSize>\n");
	string bvector;
	int nTropicalCones=0;
	int someIndex=0;
	IntegerVector someInteriorPoint;
	for(int j=0;j<tempCones.size();j++)
	  {
	    //	if(j==10)break;

	  if(tempCones[j]==i)
	    {
	      nTropicalCones++;
	      
	      IntegerVector interiorPoint(N);
	      for(list<int>::const_iterator k=cones[j].begin();k!=cones[j].end();k++)
		{
		  interiorPoint+=rays[*k];
		}

	      fprintf(stdout,"<orbit>\n");
	      fprintf(stdout,"<coneIndex>\n");
	      fprintf(Stdout,"%i",j);
	      fprintf(stdout,"</coneIndex>\n");
	      fprintf(stdout,"<indices>\n");
	      IntegerVector indi(cones[j].size());
	      {
		int J=0;
		for(list<int>::const_iterator k=cones[j].begin();k!=cones[j].end();k++,J++)
		  {
		    indi[J]=*k;
		  }
	      }
	      AsciiPrinter(Stdout).printVector(indi);
	      fprintf(stdout,"</indices>\n");

	      {
		IntegerVector Rvector(raysReps.size());
		for(list<int>::const_iterator k=cones[j].begin();k!=cones[j].end();k++)
		  Rvector[rayColors[*k][0]]++;
		fprintf(Stdout,"<rvector>");
		XmlPrinter(Stdout).printVector(Rvector);
		fprintf(Stdout,"</rvector>");
	      }
	      
	      fprintf(stdout,"<interiorpoint>\n");
	      AsciiPrinter(Stdout).printVector(SymmetryGroup::compose(finalPermutation,-interiorPoint));//CHANGING SIGN
	      fprintf(stdout,"</interiorpoint>\n");

	      {
		PolyhedralCone temp= groebnerCone(g,interiorPoint);
		addFacesToSymmetricComplex(complex,temp,temp.getHalfSpaces());
	      }
	      

	      {
		//		printMatroidSubdivision(regularSubdivision(hyperSimplex,interiorPoint),n);
	      }

	      fprintf(stdout,"<orbitSize>\n");
	      fprintf(Stdout,"%i",s.orbitSize(interiorPoint));
	      fprintf(stdout,"</orbitSize>\n");

	      
	      fprintf(stdout,"</orbit>\n");
	      bvector=bvectors[j];
	      someIndex=j;
	      //someInteriorPoint=interiorPoint;
	      someInteriorPoint=SymmetryGroup::compose(finalPermutation,interiorPoint);
	    }
	  }	    
	    fprintf(stdout,"<bvector>\n");
	    fprintf(Stdout,"%s",bvector.c_str());
	    fprintf(stdout,"</bvector>\n");

	    fprintf(stdout,"<pictureIndex>\n");
	    fprintf(Stdout,"%i",someIndex);
	    fprintf(stdout,"</pictureIndex>\n");
	    
	    fprintf(stdout,"<nTropicalCones>\n");
	    fprintf(Stdout,"%i",nTropicalCones);
	    fprintf(stdout,"</nTropicalCones>\n");
	    
	    {
	      extern list<string> treeStrings(int d, int n, IntegerVector const &v);
	      list<string> trees=treeStrings(d,n,someInteriorPoint);
	      fprintf(stdout,"<trees>\n");
	      for(list<string>::const_iterator i=trees.begin();i!=trees.end();i++)
		fprintf(stdout,"<tree>%s</tree>\n",i->c_str());
	      fprintf(stdout,"</trees>\n");
	    }
	/*if(0)
	  {
	    {
	      FILE *f=fopen("TEMPIN","w");
	      AsciiPrinter(f).printPolynomialRing(g.getRing());
	      AsciiPrinter(f).printPolynomialSet(g);
	      AsciiPrinter(f).printVector(interiorPoint);
	      fclose(f);
	      system("./gfan_initialforms --ideal --pair < TEMPIN |./gfan_groebnercone --xml --pair --asfan >TEMPOUT");
	      
	      f=fopen("TEMPOUT","r");
	      assert(f);
	      int c;
	      while((c=getc(f))!=EOF)putc(c,stdout);
	      fclose(f);
	    }
	    
	    
	    FILE *f=fopen("TEMPIN","w");
	    AsciiPrinter(f).printVector(interiorPoint);
	    fclose(f);
	    char command[1024];
	    
	    sprintf(command,"./gfan _tropicallinearspace -d%i -n%i <TEMPIN | ./gfan _tropicalintersection >TEMPOUT",d,n);
	    system(command);
	    
	    PolymakeFile inFile2;
	    inFile2.open("TEMPOUT");
	    IntegerVector fvector=inFile2.readCardinalVectorProperty("F_VECTOR");
	    IntegerVector fvectorB=inFile2.readCardinalVectorProperty("F_VECTOR_BOUNDED");
	    
	    fprintf(stdout,"<fvector>\n");
	    AsciiPrinter(Stdout).printVector(fvector);	
	    fprintf(stdout,"</fvector>\n");
	    
	    fprintf(stdout,"<fvectorB>\n");
	    AsciiPrinter(Stdout).printVector(fvectorB);	
	    fprintf(stdout,"</fvectorB>\n");
	  }
	*/
	fprintf(stdout,"</subdivision>\n");  
      }
    fprintf(stdout,"<fvector>\n");
    AsciiPrinter(Stdout).printVector(complex.fvector());
    fprintf(stdout,"</fvector>\n");
    fprintf(stdout,"<fvector2>\n");
    if(n==6)
      fprintf(Stdout,"(1,65,535,1350,1005)\n");
    else if(n==7)
      fprintf(Stdout,"(1,616,13860,101185,315070,431025,211365)\n");
    fprintf(stdout,"</fvector2>\n");

    fprintf(stdout,"<variables>\n");
    if(n==7)
      fprintf(stdout,"(p012, p013, p014, p015, p016, p023, p024, p025, p026, p034, p035, p036, p045, p046, p056, p123, p124, p125, p126, p134, p135, p136, p145, p146, p156, p234, p235, p236, p245, p246, p256, p345, p346, p356, p456)");
    //      fprintf(stdout,"(p456, p356, p346, p345, p256, p246, p245, p236, p235, p234, p156, p146, p145, p136, p135, p134, p126, p125, p124, p123, p056, p046, p045, p036, p035, p034, p026, p025, p024, p023, p016, p015, p014, p013, p012)");
    else if(n==6)
      fprintf(stdout,"(p012, p013, p014, p015, p023, p024, p025, p034, p035, p045, p123, p124, p125, p134, p135, p145, p234, p235, p245, p345)");
    fprintf(stdout,"</variables>\n");
    fprintf(stdout,"<rays>\n");
    fprintf(stdout,"<m>\n");
    IntegerVectorList temp=(-1*rays).getRows();//CHANGING SIGN
    {
      int I=0;
      for(IntegerVectorList::const_iterator i=temp.begin();i!=temp.end();i++,I++)
	{
	  fprintf(stdout,"<v>");
	  XmlPrinter(stdout).printVector(SymmetryGroup::compose(finalPermutation,*i));
	  fprintf(stdout,"<color>%s</color>",htmlColor(rayColors[I][0]));
	  fprintf(stdout,"</v>");
      }
    }
    fprintf(stdout,"</m>\n");
    fprintf(stdout,"</rays>\n");

    fprintf(stdout,"<lower>");
    for(int dim=(n==7)?12:9;dim>=n;dim--)
      {
	fprintf(stdout,"<cones dim=\"%i\">",dim);
	
	for(set<SymmetricComplex::Cone>::const_iterator i=complex.cones.begin();i!=complex.cones.end();i++)
	  if(i->dimension==dim)
	    {
	      fprintf(stdout,"<cone>");
	      fprintf(stdout,"(");
	      for(int j=0;j<i->indices.size();j++)
		{
		  if(j!=0)fprintf(stdout,",");
		  fprintf(stdout,"%i",i->indices[j]);
		}
	      fprintf(stdout,")");

	      IntegerVector interiorPoint(N);
	      for(int j=0;j<i->indices.size();j++)
		{
		  interiorPoint+=rays[i->indices[j]];
		}
	      fprintf(stdout,"<interiorpoint>\n");
	      AsciiPrinter(Stdout).printVector(SymmetryGroup::compose(finalPermutation,-interiorPoint));//CHANGING SIGN
	      fprintf(stdout,"</interiorpoint>\n");
	      

	      fprintf(stdout,"<orbitSize>\n");
	      fprintf(Stdout,"%i",s.orbitSize(i->sortKey));
	      fprintf(stdout,"</orbitSize>\n");

	      fprintf(stdout,"</cone>");
	    }
	fprintf(stdout,"</cones>");
      }
	  fprintf(stdout,"</lower>");


    fprintf(stdout,"</orbits>\n");
    

    AsciiPrinter(stderr).printVector(complex.fvector());

    return 0;
  }
};

static GrassmannDataApplication theApplication;
