/* This application is just for the project with Sven, Bernd and Michael */

#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "lp.h"
#include "gfanapplication.h"
#include "polyhedralcone.h"

#include "polymakefile.h"
#include "determinant.h"
#include "subspace.h"
#include "triangulation.h"

#include "symmetry.h"
#include "regularsubdivision.h"
#include "symmetriccomplex.h"

#include "buchberger.h"
#include "polyhedralfan.h"
#include "wallideal.h"
#include "tropical2.h"

#include "field_zmodpz.h"

#include "tropical.h"

class GrassmannData2Application : public GFanApplication
{
  StringOption inputOption;
  //  StringOption outputOption;
  IntegerOption dOption;
  IntegerOption nOption;
  IntegerOption cOption;
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program ........\n";
  }
  GrassmannData2Application():
    cOption("-c","Perform modulo test. Value is the characteristic of the field to be checked. O means no such check.",0),
    dOption("-d","Specify d.",3),
    nOption("-n","Specify n.",7),
    inputOption("-i","Specify the name of the input file.","examples/G3_7/grassmann3_7.relativeInteriorPoints")
    //    outputOption("-o","Specify the name of the output file.","polymake.out2")
  {
    registerOptions();
  }    

  char *name()
  {
    return "_grassmanndata2";
  }

  void printIntList(list<int> const &v)
  {
    FILE *f=Stderr;
    fprintf(f,"{");
    for(list<int>::const_iterator i=v.begin();i!=v.end();i++)
      {
	if(i!=v.begin())fprintf(f," ");
	fprintf(f,"%i",*i);
      }
    fprintf(f,"}\n");
  }
  void printIntListList(list<list<int> > const &l)
  {
    for(list<list<int> >::const_iterator i=l.begin();i!=l.end();i++)
      printIntList(*i);
  }
  void printIntSet(set<int> const &v)
  {
    FILE *f=Stderr;
    fprintf(f,"{");
    for(set<int>::const_iterator i=v.begin();i!=v.end();i++)
      {
	if(i!=v.begin())fprintf(f," ");
	fprintf(f,"%i",*i);
      }
    fprintf(f,"}\n");
  }
  void printIntSetSet(set<set<int> > const &l)
  {
    for(set<set<int> >::const_iterator i=l.begin();i!=l.end();i++)
      printIntSet(*i);
  }

  int numberOfIntegers(set<set<int > > const &div)
  {
    int ret=0;
    for(set<set<int > >::const_iterator i=div.begin();i!=div.end();i++)
      {
	ret+=i->size();
      }
    return ret;
  }
  set<set<int> > permutedSubdivision(set<set<int> > const &div, IntegerVector const &perm)
  {
    set<set<int > > ret;

    for(set<set<int> >::const_iterator i=div.begin();i!=div.end();i++)
      {
	set<int> temp;
	for(set<int>::const_iterator j=i->begin();j!=i->end();j++)
	  {
	    temp.insert(perm[*j]);
	  }

	ret.insert(temp);
      }

    return ret;
  }
  int subdivisionOrbit(list<set<set<int > > > &subdivisions, SymmetryGroup const &s, set<set<int> > const &div)
  {
    int I=0;
    for(list<set<set<int > > >::const_iterator i=subdivisions.begin();i!=subdivisions.end();i++,I++)
      {
	if(numberOfIntegers(*i)==numberOfIntegers(div))
	  for(SymmetryGroup::ElementContainer::const_iterator j=s.elements.begin();j!=s.elements.end();j++)
	    {
	      if(permutedSubdivision(div,*j)==*i)
		{
		  return I;
		}
	    }
      }
    subdivisions.push_back(div);
    return I;
  }

  void printBasis(int j, FILE *f)
  {
	    fprintf(f,"<basis>\n");
	    
    int d=7;
    int choose=3;
    vector<int> I;
    for(int i=0;i<d-choose;i++)I.push_back(0);
    for(int i=0;i<choose;i++)I.push_back(1);

    for(int i=0;i<j;i++)
      next_permutation(I.begin(),I.end());

    for(int i=0;i<I.size();i++)
      if(I[i])fprintf(f,"%i ",i);
	    fprintf(f,"</basis>\n");
  }

  void printMatroidSubdivision(set<set<int> > const &mList)
  {
    FILE *f=stdout;
    fprintf(f,"<matroidsubdivision>");
    for(set<set<int> >::const_iterator i=mList.begin();i!=mList.end();i++)
      {
	fprintf(f,"<matroid>\n");
	for(set<int>::const_iterator j=i->begin();j!=i->end();j++)
	  {
	    printBasis(*j,f);
	  }
	fprintf(f,"</matroid>\n");
      }
    fprintf(f,"</matroidsubdivision>\n");
  }

  PolyhedralCone groebnerCone(PolynomialSet g, IntegerVector const &w)
  {
    WeightReverseLexicographicTermOrder T(w);
    buchberger(&g,T);

    IntegerVectorList equalities=wallInequalities(initialForms(g,w));
    IntegerVectorList normals=algebraicTest(wallInequalities(g),g);

    PolyhedralCone c(normals,equalities,g.getRing().getNumberOfVariables());
    c.canonicalize();

    return c;
  }

  int permutedRayIndex(IntegerMatrix const &rays, IntegerVector const &perm, int i)
  {
    IntegerVector temp=SymmetryGroup::compose(perm,rays[i]);
    for(int j=0;j<rays.getHeight();j++)
      if(rays[j]==temp)return j;
    assert(0);
    return 0;
  }


  IntegerVectorList computeInteriorPoints(IntegerMatrix const &rays, vector<list<int> > const &cones)
  {
    IntegerVectorList ret;
    for(vector<list<int> >::const_iterator i=cones.begin();i!=cones.end();i++)
      {
	IntegerVector interiorPoint(rays.getWidth());
	for(list<int>::const_iterator j=i->begin();j!=i->end();j++)
	  {
	    interiorPoint+=rays[*j];
	  }
	ret.push_back(interiorPoint);
      }
    return ret;
  }

  /*  void moduloTest(int p, IntegerVectorList pointsToCheck)
  {
    FieldZModPZ f(p);

    PolynomialRing R(f,subsetVariableNames("p",7,3));

    FILE *f1=fopen("examples/G3_7/grassmann3_7.relativeInteriorPoints","r");
    assert(f1);
    PolynomialRing R2=FileParser(f1).parsePolynomialRing();
    PolynomialSet g=FileParser(f1).parsePolynomialSet(R);

    int I=0;
    for(IntegerVectorList::const_iterator i=pointsToCheck.begin();i!=pointsToCheck.end();i++,I++)
      {
	bool c=containsMonomial(initialIdeal(g,*i));
	fprintf(Stderr,"%i,%i\n",I,c);
      }
      }*/


  char *filenameRelativeInteriorPoints(int n)
  {
    if(n==6)
      return "grassmann3_6.relativeInteriorPoints";
       
    if(n==7)
      return "examples/G3_7/grassmann3_7.relativeInteriorPoints";
    return "UNKOWNFILE";
  }
  char *filenameWithSym(int n)
  {
    if(n==6)
      return "grassmann3_6NEWCONVENTION";
       
    if(n==7)
      return "examples/grassmann3_7.withsym";
    return "UNKOWNFILE";
  }
  char *filenameBVectors(int n)
  {
    if(n==6)
      return "grassmann3_6.bvectors";
       
    if(n==7)
      return "examples/G3_7/bvectors";
    return "UNKOWNFILE";
  }
  char *filenameHyperSimplex(int n)
  {
    if(n==6)
      return "hypersimplex3_6";
       
    if(n==7)
      return "examples/G3_7/hypersimplex";
    return "UNKOWNFILE";
  }
  int nChoose3(int n)
  {
    if(n==6)return 20;
    if(n==7)return 35;
    assert(0);
    return 0;
  }

    IntegerVector representative(IntegerVector const &v, SymmetryGroup const &s)
    {
      IntegerVector ret=v;
      for(SymmetryGroup::ElementContainer::const_iterator k=s.elements.begin();k!=s.elements.end();k++)
	{
	  if(SymmetryGroup::compose(*k,v)<ret)ret=SymmetryGroup::compose(*k,v);
	}
      return ret;
    }


  int main()
  {
    int d=dOption.getValue();
    int n=nOption.getValue();

    PolymakeFile inFile;
    fprintf(Stderr,"Test\n");
    inFile.open(filenameRelativeInteriorPoints(n));
    //    inFile.open(inputOption.getValue());
    fprintf(Stderr,"Test\n");


    int N=inFile.readCardinalProperty("AMBIENT_DIM");
    //int d=inFile.readCardinalProperty("DIM");
    //int nInteriorPoints=inFile.readCardinalProperty("N_INTERIOR_POINTS");

    int nRays=inFile.readCardinalProperty("N_RAYS");
    IntegerMatrix rays=inFile.readMatrixProperty("RAYS",nRays,N);


    vector<list<int> > cones=inFile.readMatrixIncidenceProperty("ALL_ORBITS");

    /*    if(cOption.getValue())
    {
      moduloTest(cOption.getValue(),computeInteriorPoints(rays,cones));
      return 0;
    }
    */

    FILE *f1=fopen(filenameWithSym(n),"r");
    //    FILE *f1=fopen("examples/grassmann3_7.withsym","r");
    assert(f1);
    PolynomialSet g=FileParser(f1).parsePolynomialSetWithRing();

    SymmetryGroup s(g.numberOfVariablesInRing());
    IntegerVectorList generators=FileParser(f1).parseIntegerVectorList();
	s.computeClosure(generators);


    fclose(f1);
    /*
    FILE *f3=fopen("examples/G3_7/bvectors","r");
    assert(f3);
    vector<string> bvectors(125);
    for(int i=0;i<125;i++)
      {
	char line[4096];
	fgets(line,4095,f3);
	bvectors[i]=string(line);
      }
    fclose(f3);
    */

    FILE *f2=fopen(filenameHyperSimplex(n),"r");
    //    FILE *f2=fopen("examples/G3_7/hypersimplex","r");
    assert(f2);
    IntegerMatrix hyperSimplex=rowsToIntegerMatrix(FileParser(f2).parseIntegerVectorList());
    fclose(f2);

    //    int d=dOption.getValue();
    //  int n=nOption.getValue();


    //    nInteriorPoints=2;

    //    fprintf(Stderr,"%i %i\n",n,nRays);

    //    IntegerMatrix interiorPoints=inFile.readMatrixProperty("INTERIOR_POINTS",nInteriorPoints,N);


    SymmetricComplex complex(nChoose3(n),rays.getRows(),s);
    SymmetricComplex secondaryComplex(nChoose3(n),rays.getRows(),s);
    //      SymmetricComplex complex(35,rays.getRows(),s);
    //        SymmetricComplex secondaryComplex(35,rays.getRows(),s);
      PolyhedralFan secondaryFan(nChoose3(n));
      for(int i=0;i<cones.size();i++)
	{
	  set<int> temp;
	  IntegerVectorList normals;
	  for(list<int>::const_iterator j=cones[i].begin();j!=cones[i].end();j++)
	    {
	      temp.insert(*j);
	    }
	  //	  complex.insert(SymmetricComplex::Cone(temp,13,1,true,complex));
	}


    /* Make sure that representatives of rays are in the same subspace. */
      IntegerMatrix rays2=rays;
      IntegerMatrix linealitySpace=inFile.readMatrixProperty("LINEALITY_SPACE",7,nChoose3(n));

      fprintf(Stderr,"WIDHT:%i HEIGHT:%i\n",linealitySpace.getWidth(),linealitySpace.getHeight());

      Subspace l(linealitySpace.getRows(),linealitySpace.getWidth());
      fprintf(Stderr,"%i %i\n",rays[0].size(),l.ambientDimension());
      for(int i=0;i<rays.getHeight();i++)
	rays2[i]=l.canonicalizeVector(rays[i]);


    fprintf(stdout,"<orbits>\n");

    list<set<set<int> > > subdivisions;

    vector<int> tempCones(cones.size());

    vector<set<set<int> > > subdivisionsGroebner(cones.size());
    for(int i=0;i<cones.size();i++)
      {
	//	if(i==3)break;
	fprintf(Stderr,"%i\n",i);


	IntegerVector interiorPoint(N);
	for(list<int>::const_iterator j=cones[i].begin();j!=cones[i].end();j++)
	  {
	    interiorPoint+=rays[*j];
	  }
	subdivisionsGroebner[i]=regularSubdivision(hyperSimplex,interiorPoint);
	int subdivision=subdivisionOrbit(subdivisions,s,subdivisionsGroebner[i]);
	
	tempCones[i]=subdivision;
      }

    list<set<set<int> > >::const_iterator I=subdivisions.begin();
    for(int i=0;i<subdivisions.size();i++,I++)
      {
	fprintf(Stderr,"SUBDIVISION:\n");
	printIntSetSet(*I);

	fprintf(Stderr,"%i\n",i);
	fprintf(stdout,"<subdivision>\n");
	fprintf(stdout,"<subdivisionIndex>\n");
	fprintf(Stdout,"%i",i);
	fprintf(stdout,"</subdivisionIndex>\n");


	string bvector;
	int nTropicalCones=0;
	int someIndex=0;
	set<int> indices;
	for(int j=0;j<tempCones.size();j++)
	  {
	    //  	if(j==3)break;

	  if(tempCones[j]==i)
	    {
	      fprintf(stderr,"Processing cone:\n");
	      for(list<int>::const_iterator k=cones[j].begin();k!=cones[j].end();k++)
		fprintf(stderr,"%i ",*k);
	      fprintf(stderr,"\n");
	      for(SymmetryGroup::ElementContainer::const_iterator l=s.elements.begin();l!=s.elements.end();l++)
		{
		  //		  if(permutedSubdivision(subdivisionsGroebner[j],*l)==*I)
		  if(permutedSubdivision(*I,*l)==subdivisionsGroebner[j])
		    {
		      fprintf(stderr,"Adding permuted version:\n");
	IntegerVector interiorPoint(N);

		      for(list<int>::const_iterator k=cones[j].begin();k!=cones[j].end();k++)
			{
			  fprintf(stderr,"%i ",permutedRayIndex(rays,*l,*k));
			  indices.insert(permutedRayIndex(rays,*l,*k));
			  fprintf(stderr,"\n");
			  interiorPoint+=rays[*k];
			}
		      AsciiPrinter(Stderr).printVector(interiorPoint);
		      //		      return I;
		    }
		}


	    }
	  }
	IntegerVectorList generators;
	fprintf(stderr,"Total set of generators:\n");
	for(set<int>::const_iterator k=indices.begin();k!=indices.end();k++)
	  {
	    fprintf(Stderr,"%i ",*k);
	    generators.push_back(rays2[*k]);
	  }
	fprintf(Stderr,"\n");


	/*	set<int> trueIndices;
	IntegerVectorList::const_iterator K=generators.begin();
	for(set<int>::const_iterator k=indices.begin();k!=indices.end();k++,K++)
	  if(fastIsFacet(generators,K))
	    trueIndices.insert(*k);
	//	secondaryComplex.insert(SymmetricComplex::Cone(trueIndices,rankOfMatrix(generators)+7,1,true,secondaryComplex));
void addFacesToSymmetricComplex(SymmetricComplex &c, set<int> const &indices, IntegerVectorList const &facetCandidates, int dimension, int multiplicity);
	addFacesToSymmetricComplex(secondaryComplex, trueIndices, IntegerVectorList const &facetCandidates);
	*/



	PolyhedralCone C(generators,linealitySpace.getRows(),nChoose3(n));
	fprintf(Stderr,"Finding facets\n");
	C.findFacets();
	fprintf(Stderr,"Canonicalizing\n");
	C.canonicalize();
	
	fprintf(Stderr,"Double description\n");
	IntegerVectorList inequalities=C.extremeRays();
	
	IntegerVectorList lspace=linealitySpace.getRows();
	fprintf(Stderr,"lspace:\n");
	AsciiPrinter(Stderr).printVectorList(lspace);
	fprintf(Stderr,"generators:\n");
	AsciiPrinter(Stderr).printVectorList(generators);
	for(IntegerVectorList::const_iterator i=lspace.begin();i!=lspace.end();i++)generators.push_back(*i);
	fprintf(Stderr,"RANK:%i\n",rankOfMatrix(generators));
	AsciiPrinter(Stderr).printVectorList(generators);
	IntegerVectorList empty;
	PolyhedralCone C2(empty,generators,nChoose3(n));
	fprintf(Stderr,"Dualizing\n");
	PolyhedralCone C3(inequalities,C2.dualCone().getEquations(),nChoose3(n));
	fprintf(Stderr,"Canonicalizing\n");
	C3.canonicalize();
	secondaryFan.insert(C3);
	fprintf(Stderr,"inserted.\n");
      }

    //    AsciiPrinter(Stderr).printVector(secondaryFan.fvector());
    AsciiPrinter P(Stdout);
    secondaryFan.printWithIndices(&P,FPF_default|FPF_conesCompressed|FPF_group,&s);
    //secondaryFan.printWithIndices(&P,false,&s,true);

    //    P.printString(secondaryComplex.toString(secondaryComplex.getMinDim(),secondaryComplex.getMaxDim(), false, true));


    return 0;


    for(int i=0;i<subdivisions.size();i++)
      {
	fprintf(Stderr,"%i\n",i);
	fprintf(stdout,"<subdivision>\n");
	fprintf(stdout,"<subdivisionIndex>\n");
	fprintf(Stdout,"%i",i);
	fprintf(stdout,"</subdivisionIndex>\n");
	string bvector;
	int nTropicalCones=0;
	int someIndex=0;
	for(int j=0;j<tempCones.size();j++)
	  {
	    //	if(j==10)break;

	  if(tempCones[j]==i)
	    {
	      nTropicalCones++;
	      
	      IntegerVector interiorPoint(N);
	      for(list<int>::const_iterator k=cones[j].begin();k!=cones[j].end();k++)
		{
		  interiorPoint+=rays[*k];
		}

	      fprintf(stdout,"<orbit>\n");
	      fprintf(stdout,"<coneIndex>\n");
	      fprintf(Stdout,"%i",j);
	      fprintf(stdout,"</coneIndex>\n");
	      fprintf(stdout,"<indices>\n");
	      IntegerVector indi(cones[j].size());
	      {
		int J=0;
		for(list<int>::const_iterator k=cones[j].begin();k!=cones[j].end();k++,J++)
		  {
		    indi[J]=*k;
		  }
	      }
	      AsciiPrinter(Stdout).printVector(indi);
	      fprintf(stdout,"</indices>\n");
	      
	      fprintf(stdout,"<interiorpoint>\n");
	      AsciiPrinter(Stdout).printVector(-interiorPoint);//CHANGING SIGN
	      fprintf(stdout,"</interiorpoint>\n");

	      {
		PolyhedralCone temp= groebnerCone(g,interiorPoint);
		addFacesToSymmetricComplex(complex,temp,temp.getHalfSpaces());
	      }
	      

	      {
		printMatroidSubdivision(regularSubdivision(hyperSimplex,interiorPoint));
	      }

	      fprintf(stdout,"<orbitSize>\n");
	      fprintf(Stdout,"%i",s.orbitSize(interiorPoint));
	      fprintf(stdout,"</orbitSize>\n");

	      
	      fprintf(stdout,"</orbit>\n");
	      //	      bvector=bvectors[j];
	      someIndex=j;
	    }
	  }	    
	/*	    fprintf(stdout,"<bvector>\n");
	    fprintf(Stdout,"%s",bvector.c_str());
	    fprintf(stdout,"</bvector>\n");
	*/
	/*	    fprintf(stdout,"<pictureIndex>\n");
	    fprintf(Stdout,"%i",someIndex);
	    fprintf(stdout,"</pictureIndex>\n");
	*/  
	    fprintf(stdout,"<nTropicalCones>\n");
	    fprintf(Stdout,"%i",nTropicalCones);
	    fprintf(stdout,"</nTropicalCones>\n");
	    


	/*if(0)
	  {
	    {
	      FILE *f=fopen("TEMPIN","w");
	      AsciiPrinter(f).printPolynomialRing(g.getRing());
	      AsciiPrinter(f).printPolynomialSet(g);
	      AsciiPrinter(f).printVector(interiorPoint);
	      fclose(f);
	      system("./gfan_initialforms --ideal --pair < TEMPIN |./gfan_groebnercone --xml --pair --asfan >TEMPOUT");
	      
	      f=fopen("TEMPOUT","r");
	      assert(f);
	      int c;
	      while((c=getc(f))!=EOF)putc(c,stdout);
	      fclose(f);
	    }
	    
	    
	    FILE *f=fopen("TEMPIN","w");
	    AsciiPrinter(f).printVector(interiorPoint);
	    fclose(f);
	    char command[1024];
	    
	    sprintf(command,"./gfan _tropicallinearspace -d%i -n%i <TEMPIN | ./gfan _tropicalintersection >TEMPOUT",d,n);
	    system(command);
	    
	    PolymakeFile inFile2;
	    inFile2.open("TEMPOUT");
	    IntegerVector fvector=inFile2.readCardinalVectorProperty("F_VECTOR");
	    IntegerVector fvectorB=inFile2.readCardinalVectorProperty("F_VECTOR_BOUNDED");
	    
	    fprintf(stdout,"<fvector>\n");
	    AsciiPrinter(Stdout).printVector(fvector);	
	    fprintf(stdout,"</fvector>\n");
	    
	    fprintf(stdout,"<fvectorB>\n");
	    AsciiPrinter(Stdout).printVector(fvectorB);	
	    fprintf(stdout,"</fvectorB>\n");
	  }
	*/
	fprintf(stdout,"</subdivision>\n");  
      }
    fprintf(stdout,"<rays>\n");
    XmlPrinter(stdout).printVectorList((-1*rays).getRows());//CHANGING SIGN
    fprintf(stdout,"</rays>\n");

    /*
    fprintf(stdout,"<lower>");
    for(int dim=12;dim>=7;dim--)
      {
	fprintf(stdout,"<cones dim=\"%i\">",dim);
	
	for(set<SymmetricComplex::Cone>::const_iterator i=complex.cones.begin();i!=complex.cones.end();i++)
	  if(i->dimension==dim)
	    {
	      fprintf(stdout,"<cone>");
	      fprintf(stdout,"(");
	      for(int j=0;j<i->indices.size();j++)
		{
		  if(j!=0)fprintf(stdout,",");
		  fprintf(stdout,"%i",i->indices[j]);
		}
	      fprintf(stdout,")");

	      IntegerVector interiorPoint(N);
	      for(int j=0;j<i->indices.size();j++)
		{
		  interiorPoint+=rays[i->indices[j]];
		}
	      fprintf(stdout,"<interiorpoint>\n");
	      AsciiPrinter(Stdout).printVector(-interiorPoint);//CHANGING SIGN
	      fprintf(stdout,"</interiorpoint>\n");
	      

	      fprintf(stdout,"<orbitSize>\n");
	      fprintf(Stdout,"%i",s.orbitSize(i->sortKey));
	      fprintf(stdout,"</orbitSize>\n");

	      fprintf(stdout,"</cone>");
	    }
	fprintf(stdout,"</cones>");
      }
	  fprintf(stdout,"</lower>");
    */

    fprintf(stdout,"</orbits>\n");
    

    AsciiPrinter(stderr).printVector(complex.fvector());

    return 0;
  }
};

static GrassmannData2Application theApplication;
