/* This application is just for the project with Sven, Bernd and Michael */
/* Calls M2 for ZZ initial ideals */
#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "lp.h"
#include "gfanapplication.h"
#include "polyhedralcone.h"

#include "polymakefile.h"
#include "determinant.h"
#include "subspace.h"
#include "triangulation.h"

#include "symmetry.h"
#include "regularsubdivision.h"
#include "symmetriccomplex.h"

#include "buchberger.h"
#include "polyhedralfan.h"
#include "wallideal.h"
#include "tropical2.h"

#include "field_zmodpz.h"

#include "tropical.h"

class GrassmannData3Application : public GFanApplication
{
  StringOption inputOption;
  //  StringOption outputOption;
  IntegerOption dOption;
  IntegerOption nOption;
  IntegerOption cOption;
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program ........\n";
  }
  GrassmannData3Application():
    cOption("-c","Perform modulo test. Value is the characteristic of the field to be checked. O means no such check.",0),
    dOption("-d","Specify d.",3),
    nOption("-n","Specify n.",7),
    inputOption("-i","Specify the name of the input file.","examples/G3_7/grassmann3_7.relativeInteriorPoints")
    //    outputOption("-o","Specify the name of the output file.","polymake.out2")
  {
    registerOptions();
  }    

  char *name()
  {
    return "_grassmanndata3";
  }


  IntegerVectorList computeInteriorPoints(IntegerMatrix const &rays, vector<list<int> > const &cones)
  {
    IntegerVectorList ret;
    for(vector<list<int> >::const_iterator i=cones.begin();i!=cones.end();i++)
      {
	IntegerVector interiorPoint(rays.getWidth());
	for(list<int>::const_iterator j=i->begin();j!=i->end();j++)
	  {
	    interiorPoint+=rays[*j];
	  }
	ret.push_back(interiorPoint);
      }
    return ret;
  }



  int main()
  {
    PolymakeFile inFile;
    fprintf(Stderr,"Test\n");
    inFile.open(inputOption.getValue());
    fprintf(Stderr,"Test\n");


    int N=inFile.readCardinalProperty("AMBIENT_DIM");
    //int d=inFile.readCardinalProperty("DIM");
    //int nInteriorPoints=inFile.readCardinalProperty("N_INTERIOR_POINTS");

    int nRays=inFile.readCardinalProperty("N_RAYS");
    IntegerMatrix rays=inFile.readMatrixProperty("RAYS",nRays,N);


    vector<list<int> > cones=inFile.readMatrixIncidenceProperty("ALL_ORBITS");

    IntegerVectorList pointsToCheck=computeInteriorPoints(rays,cones);

    printf("run \"rm m2complete\";\n");
    printf("\"m2complete\" <<endl <<close;\n");
    for(IntegerVectorList::const_iterator i=pointsToCheck.begin();i!=pointsToCheck.end();i++)
      {
	printf("W = [");
	for(int j=0;j<35;j++)
	  {
	    if(j)printf(",");
	    printf("%i",(*i)[j]);
	  }
	printf("];\n");

	printf("R = ZZ[");
	//	printf("p012,p013,p023,p123,p014,p024,p124,p034,p134,p234,p015,p025,p125,p035,p135,p235,p045,p145,p245,p345,p016,p026,p126,p036,p136,p236,p046,p146,p246,p346,p056,p156,p256,p356,p456,");
	printf("pabc,pabd,pacd,pbcd,pabe,pace,pbce,pade,pbde,pcde,pabf,pacf,pbcf,padf,pbdf,pcdf,paef,pbef,pcef,pdef,pabg,pacg,pbcg,padg,pbdg,pcdg,paeg,pbeg,pceg,pdeg,pafg,pbfg,pcfg,pdfg,pefg,");
	printf("MonomialOrder=> {Weights => W}, Global=>false];\n");

	printf("I = Grassmannian(2,6,R);\n"
	       "gens gb I\n"
	       "J = leadTerm(1,I);\n"
	       "mingens ideal J\n"
	       "\"m2temp\" << gens gb J << endl <<close\n");
	//	printf("run \"cat m2temp >> m2complete\";\n");
	printf("run \"cat m2complete m2temp >m2temp2\";\n");
	printf("run \"cp m2temp2 m2complete\";\n");
      }

    return 0;
  }
};

static GrassmannData3Application theApplication;
