#include <iostream>
#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "buchberger.h"
#include "wallideal.h"
#include "lp.h"
#include "reversesearch.h"
#include "termorder.h"
#include "ep_standard.h"
#include "ep_xfig.h"
#include "gfanapplication.h"
#include "polyhedralcone.h"
#include "polyhedralfan.h"
#include "tropical.h"
#include "tropical2.h"
#include "symmetry.h"
#include "halfopencone.h"
#include "log.h"
#include "matrix.h"


class IsTriangulationApplication : public GFanApplication
{

public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program reads in a vector configuration and a set of simplices on this set. It checks whether the set is a triangulation. The test is not complete at the moment, but is only a test that there is no geometric overlap.\n";
  }
  IsTriangulationApplication()
  {
    registerOptions();
  }
    
  char *name()
  {
    return "_istriangulation";
  }
  PolyhedralCone toPolyhedralCone(IntegerVector const &indices, IntegerMatrix const &m)
  {
    int n=m.getWidth();
    IntegerVectorList l;
    for(int i=0;i<indices.size();i++)l.push_back(m[indices[i]]);
    IntegerVectorList empty;
    PolyhedralCone c(l,empty,n);
    c.canonicalize();
    return c.dualCone();
  }

  int main()
  {
    FileParser P(Stdin);
    
    IntegerMatrix configuration=rowsToIntegerMatrix(P.parseIntegerVectorList());
    int n=configuration.getWidth();

    IntegerVectorList simplices=P.parseIntegerVectorList();
    list<PolyhedralCone> cones;
    for(IntegerVectorList::const_iterator i=simplices.begin();i!=simplices.end();i++)
      {
	cones.push_back(toPolyhedralCone(*i,configuration));
      }

    bool areOK=true;
    int I=0;
    for(list<PolyhedralCone>::const_iterator i=cones.begin();i!=cones.end();i++)
      {
	cerr<<I++<<endl;
	for(list<PolyhedralCone>::const_iterator j=cones.begin();j!=i;j++)
	  if(intersection(*i,*j).dimension()==n)
	    {
	      cerr<< "trouble"<<endl;
	      areOK=false;
	    }
      }	  
    cout<<areOK<<endl;

    return 0;
  }
};

static IsTriangulationApplication theApplication;
