#include <sstream>
#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "lp.h"
#include "gfanapplication.h"
#include "polyhedralcone.h"
#include "tropical2.h"

#include "polymakefile.h"
#include "determinant.h"
#include "subspace.h"
#include "triangulation.h"
#include "groebnerengine.h"
#include "wallideal.h"
#include "symmetry.h"
#include "nbody.h"

class MoeckelApplication : public GFanApplication
{
  StringOption inputOption;
  
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program solves ........\n";
  }
  MoeckelApplication():
    //    inputOption("-i","Specify the name of the input file.","examples/5body10sym.out"/*"examples/5bodyNEW.out2"*/)
    //    inputOption("-i","Specify the name of the input file.","examples/4bodygeneric.out"/*"examples/5bodyNEW.out2"*/)
    inputOption("-i","Specify the name of the input file.","examples/nbody/4bodyMassesACAC.out")
  {
    registerOptions();
  }    

  char *name()
  {
    return "_moeckel";
  }

  void printIntList(list<int> const &v)
  {
    FILE *f=Stderr;
    fprintf(f,"{");
    for(list<int>::const_iterator i=v.begin();i!=v.end();i++)
      {
	if(i!=v.begin())fprintf(f," ");
	fprintf(f,"%i",*i);
      }
    fprintf(f,"}\n");
  }
  void printIntListList(list<list<int> > const &l)
  {
    for(list<list<int> >::const_iterator i=l.begin();i!=l.end();i++)
      printIntList(*i);
  }


#if 0
  // N= number of bodies
  int rIndex(int i, int j, int N)const
  {
    if(j<i){i^=j;j^=i;i^=j;}
    int r=N;
    for(int I=0;I<N;I++)
      for(int J=I+1;J<N;J++)
	{
	  if((I==i)&&(J==j))return r;
	  r++;
	}
    assert(0);
  }
  Polynomial S(PolynomialRing const &r, int N, int i, int j)
  {
    //multiplied by
    Polynomial ret(r);
    int n=r.getNumberOfVariables();
    

    ret+=Term(r.getField().zHomomorphism(-1),Monomial(r,IntegerVector(n)));
    ret+=Term(r.getField().zHomomorphism(1),Monomial(r,-3*IntegerVector::standardVector(n,rIndex(i,j,N))));

    return ret;
  }

  Polynomial rPolynomial(PolynomialRing const &r, int N, int i, int j)
  {
    if(i==j)return Polynomial(r);
    return Term(r.getField().zHomomorphism(1),Monomial(r,2*IntegerVector::standardVector(r.getNumberOfVariables(),rIndex(i,j,N))));
  }
  Polynomial AlbouyChencinerEquation(PolynomialRing const &r, int N, int i, int j)
  {
    int n=r.getNumberOfVariables();
    Polynomial ret(r);
    for(int k=0;k<N;k++)
      {
	if(i!=k)ret+=Term(r.getField().zHomomorphism(1),Monomial(r,IntegerVector::standardVector(n,k)))*
	  (
	   S(r,N,i,k)*(
		       rPolynomial(r,N,j,k)
		       -rPolynomial(r,N,i,k)
		       -rPolynomial(r,N,i,j)
		       ));

	if(j!=k)ret+=Term(r.getField().zHomomorphism(1),Monomial(r,IntegerVector::standardVector(n,k)))*
	  (
	   S(r,N,j,k)*(
		       rPolynomial(r,N,i,k)
		       -rPolynomial(r,N,j,k)
		       -rPolynomial(r,N,i,j)
	      )
	   );
      }
    ret.saturate();
    return ret;
  }
  Polynomial AlbouyChencinerEquationSimplified(PolynomialRing const &r, int N, int i, int j)
  {
    int n=r.getNumberOfVariables();
    Polynomial ret(r);
    for(int k=0;k<N;k++)
      {
	if(i!=k)ret+=//Term(r.getField().zHomomorphism(1),Monomial(r,IntegerVector::standardVector(n,k)))*
	  (
	   S(r,N,i,k)*(
		       rPolynomial(r,N,j,k)
		       -rPolynomial(r,N,i,k)
		       -rPolynomial(r,N,i,j)
		       ));
	/*
	if(j!=k)ret+=Term(r.getField().zHomomorphism(1),Monomial(r,IntegerVector::standardVector(n,k)))*
	  (
	   S(r,N,j,k)*(
		       rPolynomial(r,N,i,k)
		       -rPolynomial(r,N,j,k)
		       -rPolynomial(r,N,i,j)
	      )
	      );*/
      }
    ret.saturate();
    return ret;
  }
  vector<string> variableNames(int N)
  {
    vector<string> ret;

    for(int i=0;i<N;i++)
      {
	stringstream s;
	s<<"m"<<i+1;
	ret.push_back(s.str());
      }

    for(int i=0;i<N;i++)
      for(int j=i+1;j<N;j++)
	{
	  stringstream s;
	  s<<"r"<<i+1<<j+1;
	  ret.push_back(s.str());
	}

    return ret;
  }
  PolynomialSet AlbouyChencinerEquations(int N)
  {
    PolynomialRing r(Q,variableNames(N));

    PolynomialSet ret(r);
    for(int i=0;i<N;i++)
      for(int j=i+1;j<N;j++)
	ret.push_back(AlbouyChencinerEquation(r,N,i,j));
    return ret;		      
  }

  PolynomialSet AlbouyChencinerEquationsSimplified(int N)
  {
    PolynomialRing r(Q,variableNames(N));

    PolynomialSet ret(r);
    for(int i=0;i<N;i++)
      for(int j=0;j<N;j++)
	if(i!=j)
	  ret.push_back(AlbouyChencinerEquationSimplified(r,N,i,j));
    return ret;		      
  }
#endif

  /*  PolynomialSet DziobekEquations(PolynomialRing const &r)
  {
    StringParser P("{r243*r353 - r233*r243*r353 - r233*r453 + r233*r243*r453 +r233*r353*r453 - r243*r353*r453,"
		   "   r253*r343 - r233*r253*r343 -  r233*r453 + r233*r253*r453 + r233*r343*r453 - r253*r343*r453,"
		   " -r253*r343 + r243*r253*r343 + r243*r353 - r243*r253*r353 - r243*r343*r353 + r253*r343*r353,"
		   " r143*r353 - r133*r143*r353 - r133*r453 + r133*r143*r453 + r133*r353*r453 - r143*r353*r453,"
		   " r153*r343 - r133*r153*r343 - r133*r453 + r133*r153*r453 + r133*r343*r453 - r153*r343*r453,"
		   " -r153*r343 + r143*r153*r343 +  r143*r353 - r143*r153*r353 - r143*r343*r353 + r153*r343*r353,"
		   " r143*r253 - r123*r143*r253 - r123*r453 + r123*r143*r453 +  r123*r253*r453 - r143*r253*r453,"
		   " r153*r243 - r123*r153*r243 -  r123*r453 + r123*r153*r453 + r123*r243*r453 - r153*r243*r453,"
		   " -r153*r243 + r143*r153*r243 + r143*r253 - r143*r153*r253 -  r143*r243*r253 + r153*r243*r253,"
		   " r133*r253 - r123*r133*r253 -  r123*r353 + r123*r133*r353 + r123*r253*r353 - r133*r253*r353,"
		   " r153*r233 - r123*r153*r233 - r123*r353 + r123*r153*r353 +  r123*r233*r353 - r153*r233*r353,"
		   " -r153*r233 + r133*r153*r233 +  r133*r253 - r133*r153*r253 - r133*r233*r253 + r153*r233*r253,"
		   " r133*r243 - r123*r133*r243 - r123*r343 + r123*r133*r343 +  r123*r243*r343 - r133*r243*r343,"
		   " r143*r233 - r123*r143*r233 -  r123*r343 + r123*r143*r343 + r123*r233*r343 - r143*r233*r343,"
		   " -r143*r233 + r133*r143*r233 + r133*r243 - r133*r143*r243 -  r133*r233*r243 + r143*r233*r243}");

    return P.parsePolynomialSet(r);
    }*/

  PolynomialSet DziobekEquations(PolynomialRing const &r)
  {
    StringParser P("{r143r233-r123r343-r123r143r233+r123r143r343+r123r233r343-r143r233r343,"
		   "r133r243-r123r343-r123r133r243+r123r133r343+r123r243r343-r133r243r343,"
		   "r143r233-r133r243-r133r143r233+r133r143r243+r133r233r243-r143r233r243}");

    /*    StringParser P("{r243*r353 - r233*r243*r353 - r233*r453 + r233*r243*r453 +r233*r353*r453 - r243*r353*r453,"
		   "   r253*r343 - r233*r253*r343 -  r233*r453 + r233*r253*r453 + r233*r343*r453 - r253*r343*r453,"
		   " -r253*r343 + r243*r253*r343 + r243*r353 - r243*r253*r353 - r243*r343*r353 + r253*r343*r353,"
		   " r143*r353 - r133*r143*r353 - r133*r453 + r133*r143*r453 + r133*r353*r453 - r143*r353*r453,"
		   " r153*r343 - r133*r153*r343 - r133*r453 + r133*r153*r453 + r133*r343*r453 - r153*r343*r453,"
		   " -r153*r343 + r143*r153*r343 +  r143*r353 - r143*r153*r353 - r143*r343*r353 + r153*r343*r353,"
		   " r143*r253 - r123*r143*r253 - r123*r453 + r123*r143*r453 +  r123*r253*r453 - r143*r253*r453,"
		   " r153*r243 - r123*r153*r243 -  r123*r453 + r123*r153*r453 + r123*r243*r453 - r153*r243*r453,"
		   " -r153*r243 + r143*r153*r243 + r143*r253 - r143*r153*r253 -  r143*r243*r253 + r153*r243*r253,"
		   " r133*r253 - r123*r133*r253 -  r123*r353 + r123*r133*r353 + r123*r253*r353 - r133*r253*r353,"
		   " r153*r233 - r123*r153*r233 - r123*r353 + r123*r153*r353 +  r123*r233*r353 - r153*r233*r353,"
		   " -r153*r233 + r133*r153*r233 +  r133*r253 - r133*r153*r253 - r133*r233*r253 + r153*r233*r253,"
		   " r133*r243 - r123*r133*r243 - r123*r343 + r123*r133*r343 +  r123*r243*r343 - r133*r243*r343,"
		   " r143*r233 - r123*r143*r233 -  r123*r343 + r123*r143*r343 + r123*r233*r343 - r143*r233*r343,"
		   " -r143*r233 + r133*r143*r233 + r133*r243 - r133*r143*r243 -  r133*r233*r243 + r143*r233*r243}");
    */
    return P.parsePolynomialSet(r);
    }


  PolynomialSet saturationHomog(PolynomialSet const &g, int i)
  {
    WeightReverseLexicographicTermOrder T(-1*IntegerVector::standardVector(g.getRing().getNumberOfVariables(),i));
    PolynomialSet g2=GE_groebnerBasis(g,T,false);
    g2.saturate(i);
    return g2;
  }

  PolynomialSet saturateAndIntersect(PolynomialSet const &g, int N)
  {
    PolynomialRing r(g.getRing().getField(),g.getRing().getNumberOfVariables()+1);
    PolynomialSet g2=g.homogenization(r);

    for(int i=0;i<g.getRing().getNumberOfVariables();i++)
      //for(int i=0;i<g.getRing().getNumberOfVariables();i++)
      g2=saturationHomog(g2,i);

    /*    WeightReverseLexicographicTermOrder T(concatenation(IntegerVector::allOnes(N),4*IntegerVector::allOnes(1+N*(N-1)/2)));
    g2=GE_groebnerBasis(g2,T,false);
    */
    PolynomialRing r3(Q,0);
    PolynomialSet g3=g2.polynomialRingIntersection(r3);

    return g3;
  }

		/*		g.push_back(StringParser("1-m1").parsePolynomial(g.getRing()));
		g.push_back(StringParser("1-m2").parsePolynomial(g.getRing()));
		g.push_back(StringParser("1-m3").parsePolynomial(g.getRing()));
		g.push_back(StringParser("1-m4").parsePolynomial(g.getRing()));
		*/
		//		g.push_back(StringParser("1-m1").parsePolynomial(g.getRing()));
		/*			  g.push_back(StringParser("1-r12r13r14r23r24r34").parsePolynomial(g.getRing()));
		

	WeightReverseLexicographicTermOrder T(concatenation(IntegerVector::allOnes(N),4*IntegerVector::allOnes(N*(N-1)/2)));
	PolynomialSet g2=GE_groebnerBasis(g,T,false);

	PolynomialRing r3(Q,N);
	PolynomialSet g3=g2.polynomialRingIntersection(r3);
		
	P<<"Pseudo intersection:\n";
	P<<g3;
		*/

  int main()
  {
    int N=4;
    LpSolver::printList(Stderr);
    lpSetSolver("cddgmp");

    AsciiPrinter P(Stderr);

    PolymakeFile inFile;
    inFile.open(inputOption.getValue());

    int n=inFile.readCardinalProperty("AMBIENT_DIM");
    int nRays=inFile.readCardinalProperty("N_RAYS");

    IntegerMatrix rays=inFile.readMatrixProperty("RAYS",nRays,n);

    vector<list<int> > cones=inFile.readMatrixIncidenceProperty("CONES_COMPRESSED");
    IntegerVectorList r;
    
    PolynomialSet ac=AlbouyChencinerEquations(N,true);

    PolynomialSet D=DziobekEquations(ac.getRing());

    PolynomialSet ac2=AlbouyChencinerEquations(N,true,false);



    PolynomialSet all=ac;

    for(PolynomialSet::const_iterator i=ac2.begin();i!=ac2.end();i++)all.push_back(*i);
    for(PolynomialSet::const_iterator i=D.begin();i!=D.end();i++)all.push_back(*i);


    list<PolyhedralCone> theCones;

    for(int i=0;i<cones.size();i++)
      {
	cerr<<"-------------------"<<endl;
	IntegerVector v(n);
	IntegerVectorList m;
	for(list<int>::const_iterator j=cones[i].begin();j!=cones[i].end();j++)
	  {
	    v+=rays[*j];
	    m.push_back(rays[*j]);
	  }
	r.push_front(v);
	//	r.push_back(v);

	cerr<<i<<"/"<<cones.size()<<endl;
	P<<m<<"\n";


	IntegerMatrix M=rowsToIntegerMatrix(m,n);
	M=M.submatrix(0,0,M.getHeight(),N);
	cerr << "Rank " << rank(M)<< endl;


	
	  IntegerVectorList m2=m;
	  m2.push_back(concatenation(IntegerVector::allOnes(N),IntegerVector(n-N)));
	  IntegerMatrix M2=rowsToIntegerMatrix(m2,n);
	  cerr << "Rank of big matrix:" << rank(M2)<< endl;
	



	  if(rank(M)>=3 && rank(M2)>=5)// && i!=641 && i!=680 && i!=1115 && i!=1633 && i!=1743)
	  {
	    PolynomialSet g=initialForms(all,v.subvector(0,10));

	    PolynomialRing r(Q,N*(N-1)/2);
	    g=g.embeddedInto(r);

	P<<"Pseudo intersection:\n";
	PolynomialSet g2=saturateAndIntersect(g,N);
	P<<g2;

	if(g2.size()==0)
	  {
	    cerr<<"trouble?"<<endl;
       
	    P<< "M" <<M.getRows();

	    {
	      IntegerVectorList mm=M.getRows();
	      IntegerVectorList mmm;
	      for(IntegerVectorList::const_iterator i=mm.begin();i!=mm.end();i++)if(!i->isZero())mmm.push_back(*i);
	      P<<"extreme rays"<<fastNormals(mmm);


	    {
	    IntegerVectorList equations;
	    cerr<<"A1"<<endl;
	    equations.push_back(IntegerVector::allOnes(N));
	    cerr<<"A2"<<endl;
	    PolyhedralCone C(mm,equations,N);
	    cerr<<"A3"<<endl;
	    C.canonicalize();
	    cerr<<"A4"<<endl;
	    IntegerVectorList rays=C.extremeRays();
	    cerr<<"A5"<<endl;
	    mm.push_back(IntegerVector::allOnes(N));
	    FieldMatrix m2Q=integerMatrixToFieldMatrix(rowsToIntegerMatrix(mm),Q);
	    cerr<<"A6"<<endl;
	    IntegerVectorList equations2=fieldMatrixToIntegerMatrixPrimitive(m2Q.reduceAndComputeKernel()).getRows();
	    cerr<<"A7"<<endl;
	    PolyhedralCone C2(rays,equations2,N);
	    cerr<<"A8"<<endl;
	    C2.canonicalize();
	    C2.print(&P);
	    
	    theCones.push_back(C2);
	    }

	    }
	    P<< "M2" <<M2.getRows();



	    IntegerVectorList equations;
	    equations.push_back(concatenation(IntegerVector::allOnes(N),IntegerVector(n-N)));
	    PolyhedralCone C(m2,equations,n);
	    C.canonicalize();
	    IntegerVectorList rays=C.extremeRays();
	    FieldMatrix m2Q=integerMatrixToFieldMatrix(rowsToIntegerMatrix(m2),Q);
	    IntegerVectorList equations2=fieldMatrixToIntegerMatrixPrimitive(m2Q.reduceAndComputeKernel()).getRows();
	    for(int i=0;i<N;i++)equations2.push_back(IntegerVector::standardVector(n,i));
	    PolyhedralCone C2(rays,equations2,n);
	    C2.canonicalize();
	    C2.print(&P);
	  }
	  }
	     }


    SymmetryGroup S(N);
    IntegerVectorList generators=StringParser("{(1,0,2,3),(1,2,3,0)}").parseIntegerVectorList();
    S.computeClosure(generators);


    for(int i=0;i<10000;i++)
      {
	bool ok=false;
	IntegerVector r(N);
	for(int j=0;j<N;j++)r[j]=(rand()&63)-31;
	P<<r<<"\n";
	for(SymmetryGroup::ElementContainer::const_iterator j=S.elements.begin();j!=S.elements.end();j++)
	  {
	    IntegerVector r2=SymmetryGroup::compose(*j,r);
	    for(list<PolyhedralCone>::const_iterator k=theCones.begin();k!=theCones.end();k++)
	      if(k->contains(r2))
		{
		  ok=true;goto exit;
		}
	  }
      exit:
	if(!ok)
	  {
	    P<< r;
	    assert(0);
	  }

      }

    return 0;
  }
};

static MoeckelApplication theApplication;

