#include "vektor.h"
#include "printer.h"
#include "parser.h"
#include "gfanapplication.h"
#include "nbody.h"
#include "field_rationals.h"
#include <iostream>

class NBodyApplication : public GFanApplication
{
  IntegerOption NOption;
  SimpleOption optionWithMasses;
  SimpleOption optionSymmetric;
  SimpleOption optionDziobek;
  IntegerOption optionDeterminants;
  SimpleOption optionMM;
  SimpleOption optionSVariables;
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program computes the AC equations for the nbody problem.\n";
  }
  NBodyApplication():
    NOption("-N","Specify number of bodies.",3),
    optionWithMasses("--masses","Include mass variables."),
    optionSymmetric("--symmetric","Produce the symmetric equations"),
    optionDziobek("--dziobek","Produce the Dziobek equations"),
    optionDeterminants("--codim","Produce the determinant equations",-1),
    optionMM("--mm","Add in additional mass equation"),
    optionSVariables("-s","Treat the Sij polynomials as variables")
  {
    registerOptions();
  }
  char *name()
  {
    return "_nbody";
  }

  int main()
  {
    AsciiPrinter PP(stdout);

    PolynomialSet g=AlbouyChencinerEquations(NOption.getValue(),optionWithMasses.getValue(),optionSymmetric.getValue(),optionSVariables.getValue());

    if(optionSVariables.getValue())
      {
	PolynomialSet s=SEquations(g.getRing(),NOption.getValue(),optionWithMasses.getValue());
	for(PolynomialSet::const_iterator i=s.begin();i!=s.end();i++)g.push_back(*i);
      }

    if(optionDziobek.getValue())
      {
	PolynomialSet dz=DziobekEquations(g.getRing(),NOption.getValue(),optionWithMasses.getValue(),optionSVariables.getValue());
	for(PolynomialSet::const_iterator i=dz.begin();i!=dz.end();i++)g.push_back(*i);
      }

    if(optionDeterminants.getValue()!=-1)
      {
	PolynomialSet de=nbodyDeterminants(g.getRing(),NOption.getValue(),optionWithMasses.getValue(),1+NOption.getValue()-optionDeterminants.getValue());
	for(PolynomialSet::const_iterator i=de.begin();i!=de.end();i++)g.push_back(*i);
      }
    if(optionMM.getValue())g.push_back(massEquation(g.getRing(),NOption.getValue(),optionWithMasses.getValue()));

    PP<<g.getRing();
    PP<<g;

    return 0;
  }
};

static NBodyApplication theApplication;
