#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "gfanapplication.h"

#include "polymakefile.h"
#include "field_rationals.h"

class PolytopeToPolynomialApplication : public GFanApplication
{
  StringOption inputOption;
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program ........\n";
  }
  PolytopeToPolynomialApplication():
    inputOption("-i","Specify the name of the input file.","polymake.out")
  {
    registerOptions();
  }    

  char *name()
  {
    return "_polytopetopolynomial";
  }

  int main()
  {
    PolymakeFile inFile;
    inFile.open(inputOption.getValue());

    int n=inFile.readCardinalProperty("AMBIENT_DIM");
    int nVertices=inFile.readCardinalProperty("N_VERTICES");

    fprintf(Stderr,"%i %i\n",n,nVertices);

    IntegerMatrix vertices=inFile.readMatrixProperty("VERTICES",nVertices,n+1);

    AsciiPrinter(Stderr).printVectorList(vertices.getRows());

    PolynomialRing r=PolynomialRing(Q,n);

    Polynomial f(r);
    for(int i=0;i<vertices.getHeight();i++)
      {
	Term t(Q.zHomomorphism(1),Monomial(r,vertices[i].subvector(1,n+1)));
	f=t+f;
      }
    AsciiPrinter(Stdout).printPolynomialRing(r);
    fprintf(Stdout,"\n");
    AsciiPrinter(Stdout).printPolynomial(f);
    fprintf(Stdout,"\n");
    return 0;
  }
};

static PolytopeToPolynomialApplication theApplication;

