#include "parser.h"
#include "printer.h"
#include "saturation.h"
#include "tropical.h"
#include "gfanapplication.h"

class SaturationApplication : public GFanApplication
{
  SimpleOption optionIsHomogeneous;
public:
  const char *helpText()
  {
    return "This program computes the saturation of the input ideal with the product of the variables x_1,...,x_n. The ideal does not have to be homogeneous.\n";
  }
  SaturationApplication():
    optionIsHomogeneous("-h","Tell the program that the input is a homogeneous ideal (with homogeneous generators).\n")
  {
    registerOptions();
  }
  
  char *name()
  {
    return "_saturation";
  }

  int main()
  {
    FileParser P(Stdin);

    PolynomialSet a=P.parsePolynomialSetWithRing();

    AsciiPrinter(Stdout).printPolynomialRing(a.getRing());
    AsciiPrinter(Stdout).printNewLine();
    AsciiPrinter(Stdout).printPolynomialSet(optionIsHomogeneous.getValue()?saturatedIdeal(a):nonHomogeneousSaturation(a));
    
    return 0;
  }
};

static SaturationApplication theApplication;
