#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "buchberger.h"
#include "wallideal.h"
#include "lp.h"
#include "reversesearch.h"
#include "breadthfirstsearch.h"
#include "termorder.h"
#include "ep_standard.h"
#include "ep_xfig.h"
#include "gfanapplication.h"
#include "timer.h"
#include "log.h"
#include "matrix.h"
#include "lll.h"
#include "polyhedralfan.h"
#include "linalg.h"
#include "determinant.h"
#include "triangulation.h"
#include "intsinpolytope.h"
#include "graph.h"

#include <iostream>
#include <algorithm>

class SecondaryFanApplication : public GFanApplication
{
  /**
     TODO: Merge this class with the Triangulation class.
   */

  class Triangulation2
  {
  public:
    //    IntegerVector circuit(IntegerMatrix const &A, IntegerVector const &basis, int j);

    /**
       A projective vector configuration. The columns are the vectors.
     */
    IntegerMatrix A;
    /**
       The dual of a subdivision of the cone over A is a polyhedron with facet normals among the columns.
       The subdivision being a triangulation is equivalent to the polyhedron being simple.
       Bases is either the sets of facet normals giving rise to a poiont of the polyhedron
       OR the list of triangles of the triangulation.
     */
    set<IntegerVector> bases;
    int getN()const
    {
      return A.getWidth();
    }
    IntegerVector complement(IntegerVector const &v, int n)const
    {
      IntegerVector ret(0);
      int j=0;
      for(int i=0;i<n;i++)
	{
	  if(j>=v.size())
	    ret.push_back(i);
	  else
	    if(i==v[j])
	      j++;
	    else
	      {
		ret.push_back(i);
	      }
	}
      return ret;
    }
    PolyhedralCone secondaryCone()const
    {
      IntegerVectorList empty;
      return PolyhedralCone(facets(),empty,A.getWidth());
    }
    IntegerVector interiorPoint()const
    {
      //      IntegerVectorList empty;
      //return PolyhedralCone(inequalities(),empty,A.getWidth()).getRelativeInteriorPoint();
      //return PolyhedralCone(facets(),empty,A.getWidth()).getRelativeInteriorPoint();
      return secondaryCone().getRelativeInteriorPoint();
    }

    IntegerVector determinantInequality(list<int> const &indices) const
    {
      //  fprintf(stderr,"SET:");
      //  for(list<int>::const_iterator i=indices.begin();i!=indices.end();i++)fprintf(stderr," %i",*i);
      //  fprintf(stderr,"\n");
      int m=indices.size();
      FieldVector ret(Q,A.getWidth());
      
      int I=m;//HER
      for(list<int>::const_iterator i=indices.begin();i!=indices.end();i++,I++)
	{
	  IntegerMatrix B(m-1,A.getHeight());
	  int J=0;
	  for(list<int>::const_iterator j=indices.begin();j!=indices.end();j++)
	    if(i!=j)
	      {
		for(int k=0;k<B.getHeight();k++)
		  {
		    B[k][J]=A[k][*j];
		  }
		J++;
	      }
	  FieldMatrix B2=integerMatrixToFieldMatrix(B,Q);
	  ret[*i]=(Q.zHomomorphism(1-2*(I&1)))*B2.reduceAndComputeDeterminant();
	}
      //      AsciiPrinter T(Stderr);      ret.print(T);
      return ret.primitive();
    }
    IntegerMatrix subsetRows(IntegerMatrix const &ATransposed, IntegerVector const &cols)const
    {
      IntegerMatrix ret(cols.size(),ATransposed.getWidth());
      
      for(int i=0;i!=cols.size();i++)ret[i]=ATransposed[cols[i]];
      return ret;
    }
    FieldElement volume(IntegerVector const &v, IntegerMatrix const &ATransposed)const
    {
      IntegerMatrix B(A.getHeight(),A.getHeight());
      for(int j=0;j<v.size();j++)
	B[j]=ATransposed[v[j]];
      FieldMatrix B2=integerMatrixToFieldMatrix(B,Q);
      FieldElement det=B2.reduceAndComputeDeterminant();
      if(det.sign()<0)return -det;
      return det;
    }
    set<set<int> > coDimensionOneTriangles()const
    {
      set<set<int> > codimOne;
      for(set<IntegerVector>::const_iterator j=bases.begin();j!=bases.end();j++)
	{
	  set<int> temp;
	  for(int i=0;i<j->size();i++)
	    {
	      temp.insert((*j)[i]);
	    }
	  for(int i=0;i<j->size();i++)
	    {
	      temp.erase((*j)[i]);
	      codimOne.insert(temp);
	      temp.insert((*j)[i]);
	    }
	}
      return codimOne;
    }
    Graph edgeGraph()const
    {
      Graph ret(bases.size());

      set<set<int> > codimOne=coDimensionOneTriangles();

      for(set<set<int> >::const_iterator i=codimOne.begin();i!=codimOne.end();i++)
	{
	  list<int> triangIndices;
	  int J=0;
	  for(set<IntegerVector>::const_iterator j=bases.begin();j!=bases.end();j++,J++)
	    {
	      bool isSuperSet=true;
	      for(set<int>::const_iterator k=i->begin();k!=i->end();k++)
		{
		  bool hasBeenFound=false;
		  for(int l=0;l<j->size();l++)
		    if((*j)[l]==*k)
		      {
			hasBeenFound=true;
			break;
		      }
		  if(!hasBeenFound)
		    {
		      isSuperSet=false;
		      break;
		    }
		}
	      if(isSuperSet)
		triangIndices.push_back(J);
	    }
	  if(triangIndices.size()==2)
	    ret.addEdge(triangIndices.front(),triangIndices.back());
	}

      //      cerr << ret.toString();

      return ret;
    }
    set<int> difference(IntegerVector const &v, set<int> const &s)const
    {
      set<int> ret;
      for(int i=0;i<v.size();i++)ret.insert(v[i]);
      /*set<int> ret2;
      set_difference<set<int>::const_iterator,set<int>::const_iterator,set<int>::iterator>(ret.begin(),ret.end(),s.begin(),s.end(),ret2);
      return ret2;*/
      for(set<int>::const_iterator i=s.begin();i!=s.end();i++)ret.erase(*i);
      return ret;
    }
    /**
       Seems to compute the outer normals of the secondary cone of the triangulation.
     */

    IntegerVectorList inequalitiesFast()const
    {
      IntegerVectorList ret;

      // we get an inequality for every A-column not a vertex, i.e. not appearing in the triangulation
      for(int i=0;i<A.getWidth();i++)
	{
	  bool appears=false;
	  for(set<IntegerVector>::const_iterator j=bases.begin();j!=bases.end();j++)
	    {
	      for(int k=0;k<j->size();k++)
		if(i==(*j)[k]){appears=true;goto done;}
	    }
	done:
	  if(!appears)
	    {
	      // We now find a triangle which covers the ith column
	      set<IntegerVector>::const_iterator j;
	      int lastSign=0;
	      for(j=bases.begin();j!=bases.end();j++)
		{
		  IntegerMatrix ATransposed=A.transposed();
		  bool containsI=true;
		  IntegerMatrix subMatrix=subsetRows(A.transposed(),*j);
		  int sign=determinantSign(subMatrix.getRows());
		  for(int k=0;k<j->size();k++)
		    {
		      subMatrix[k]=ATransposed[i];
		      if(sign*determinantSign(subMatrix.getRows())<0)
			{
			  containsI=false;
			  break;
			}
		      subMatrix[k]=ATransposed[(*j)[k]];
		    }
		  lastSign=sign;
		  if(containsI)break;		    
		}
	      if(j==bases.end())
		{
		  AsciiPrinter(Stderr).printVector(A.transposed()[i]);
		}
	      assert(j!=bases.end());
	      list<int> temp;
	      for(int k=0;k<j->size();k++)temp.push_back((*j)[k]);
	      temp.push_back(i);
	      ret.push_back(lastSign* determinantInequality(temp));//Do we need to use the sign here?//HERE
	    }
	}


      // we get an inequality for every codim one simplex of the triangulation, i.e. every edge of the polyhedron

      set<set<int> > codimOne;
      for(set<IntegerVector>::const_iterator j=bases.begin();j!=bases.end();j++)
	{
	  set<int> temp;
	  for(int i=0;i<j->size();i++)
	    {
	      temp.insert((*j)[i]);
	    }
	  for(int i=0;i<j->size();i++)
	    {
	      temp.erase((*j)[i]);
	      codimOne.insert(temp);
	      temp.insert((*j)[i]);
	    }
	}
      for(set<set<int> >::const_iterator i=codimOne.begin();i!=codimOne.end();i++)
	{
	  list<int> additional;
	  for(set<IntegerVector>::const_iterator j=bases.begin();j!=bases.end();j++)
	    {
	      set<int> d=difference(*j,*i);
	      if(d.size()==1)
		{
		  additional.push_back(*d.begin());
		}
	    }
	  if(additional.size()==2)
	    {
	      list<int> temp;
	      for(set<int>::const_iterator j=i->begin();j!=i->end();j++)temp.push_back(*j);
	      list<int>::const_iterator a=additional.begin();

	      list<int> temp2=temp;
	      temp2.push_back(*a);
	      IntegerVector temp3(temp2.size());
	      list<int>::const_iterator K=temp2.begin();
	      for(int k=0;k<temp3.size();k++,K++)temp3[k]=*K;
	      //AsciiPrinter(Stderr).printVectorList(subsetRows(A.transposed(),temp3).getRows());
	      if(determinantSign(subsetRows(A.transposed(),temp3).getRows())>0)
		{
		  temp.push_back(*a);
		  a++;
		  temp.push_back(*a);
		}
	      else
		{
		  a++;
		  temp.push_back(*a);
		  a--;
		  temp.push_back(*a);
		}
	      //	      fprintf(stderr,"size %i\n",temp2.size());
	      ret.push_back(determinantInequality(temp));//HERE
	    }
	}
      return ret;
    }
    IntegerVectorList inequalities()const
    {
      int n=A.getWidth();
      IntegerVectorList ret;
      for(set<IntegerVector>::const_iterator i=bases.begin();i!=bases.end();i++)
	{
	  IntegerMatrix A2transposed(i->size()+1,A.getHeight());
	  for(int j=0;j<i->size();j++)
	    {
	      A2transposed[j]=A.column((*i)[j]);
	    }
	  IntegerVector iComplement=complement(*i,n);
	  
	  for(int k=0;k<iComplement.size();k++)
	    {
	      IntegerVector nyC;

	      A2transposed[i->size()]=A.column(iComplement[k]);
	      IntegerMatrix temp=A2transposed.transposed();
	      nyC=vectorInKernel(temp);
	      

	      IntegerVector c(n);
	      for(int j=0;j<i->size();j++)
		c[(*i)[j]]=nyC[j];
	      c[iComplement[k]]=nyC[i->size()];
	      if(c[iComplement[k]]>0)
		{
		  ret.push_back(c);
		}
	      else
		{
		  ret.push_back(-c);
		}
	    }
	}
      return ret;
    }
    int totalVolume()const
    {
      FieldElement s(Q);
      for(set<IntegerVector>::const_iterator i=bases.begin();i!=bases.end();i++)
	{
	  FieldElement vol=volume(*i,A.transposed());
	  s=s+vol;
	}
      cerr<<"retateawtat"<< s.toString()<<endl;
      return toInteger(s);
    }
    void print(Printer &p)
    {
      p.printString("Bases(");
      p.printInteger(bases.size());
      p.printString(":\n");
      FieldElement s(Q);
      for(set<IntegerVector>::const_iterator i=bases.begin();i!=bases.end();i++)
	{
	  p.printVector(*i);
	  FieldElement vol=volume(*i,A.transposed());
	  p.printString("  Vol: ");
	  p.printFieldElement(vol);
	  p.printNewLine();
	  s=s+vol;
	}
      p.printString("Total volume: ");
      p.printFieldElement(s);
      p.printNewLine();
      /*      p.printString("Circuits (inequalities):\n");
      p.printVectorList(inequalities());
      p.printString("Facets:\n");
      p.printVectorList(facets());
      p.printString("Interior point:\n");
      p.printVector(interiorPoint());
      */
    }
    IntegerVectorList facets()const
    {
      return fastNormals(inequalitiesFast());

      IntegerVectorList flipable;
      IntegerVectorList normals=wallRemoveScaledInequalities(inequalitiesFast());
      //      IntegerVectorList normals=wallRemoveScaledInequalities(inequalities());

      

      for(IntegerVectorList::iterator i=normals.begin();i!=normals.end();i++)
	{
	  //	  if(!termOrder(*i,*i-*i))
	    {
	      if(isFacet(normals,i))
		{
		  //  if(wallContainsPositiveVector(*i))
		    flipable.push_back(*i);
		}
	      else
		{
		  IntegerVectorList::iterator temp=i;
		  temp++;
		  normals.erase(i);
		  temp--;
		  i=temp;
		}
	    }
	}
      /*      AsciiPrinter Q(Stderr);
      Q.printString("Bases:\n");
      for(set<IntegerVector>::const_iterator i=bases.begin();i!=bases.end();i++)
	{
	  Q.printVector(*i);
	  Q.printNewLine();
	}

      AsciiPrinter(Stderr).printVectorList(inequalities());
      AsciiPrinter(Stderr).printVectorList(flipable);
      AsciiPrinter(Stderr).printVectorList(inequalitiesFast());
      log0 fprintf(stderr,"-----------------\n");
      */

      //log0 fprintf(stderr,"%i\n",0);
      //  log0 fprintf(stderr,"Number of facets:%i\n",flipable.size());

      return flipable;
    }
    bool isEmpty()const
    {
      return bases.empty();
    }
    void flip(IntegerVector const &normal)
    {
      AsciiPrinter P(Stderr);
      log2 print(P);
      //log0 P.printVector(normal);
      int n=normal.size();
      //      IntegerVectorList l=wallRemoveScaledInequalities(inequalities());// This is not needed - one circuit should be enough
      IntegerVectorList l;l.push_back(normal);// Let's do this instead
      for(IntegerVectorList::const_iterator i=l.begin();i!=l.end();i++)
	if(dependent(*i,normal))
	  {
	    log2 AsciiPrinter(Stderr).printVector(*i);
	    for(int k=0;k<normal.size();k++)
	      if((*i)[k]<0)
		{
		  int s1=bases.size();
		  IntegerVector i2=*i;
		  i2[k]=0;
		  bases.insert(i2.supportIndices());
		  int s2=bases.size();
		  assert(s2==s1+1);
		}
	      else if((*i)[k]>0)
		{
		  int s1=bases.size();
		  IntegerVector i2=*i;
		  i2[k]=0;
		  bases.erase(i2.supportIndices());
		  int s2=bases.size();
		  assert(s2==s1-1);
		}	      
	  }
    }
    static set<int> toSet(IntegerVector const &v)
    {
      set<int> ret;
      for(int i=0;i<v.size();i++)ret.insert(v[i]);
      return ret;
    }
    static IntegerVector toVector(set<int> const &s)
    {
      IntegerVector ret(s.size());
      int I=0;
      for(set<int>::const_iterator i=s.begin();i!=s.end();i++,I++)ret[I]=*i;
      return ret;
    }
    bool contains(IntegerVector const &v, set<int> const &s)
    {
      for(set<int>::const_iterator i=s.begin();i!=s.end();i++)
	{
	  bool found=false;
	  for(int j=0;j!=v.size();j++)
	    if(v[j]==*i)
	      {
		found=true;
		break;
	      }
	  if(!found)return false;
	}
      return true;
    }
    set<int> uni(set<int> const &a, set<int> const &b)
    {
      set<int> ret=a;
      for(set<int>::const_iterator i=b.begin();i!=b.end();i++)ret.insert(*i);
      return ret;
    }
    void printSet(set<int> const &s)
    {
      cerr<<"{";
      for(set<int>::const_iterator i=s.begin();i!=s.end();i++)
	{
	  if(i!=s.begin())cerr<<",";
	  cerr<<*i;
	}
      cerr<<"}";
    }
    void printSetSet(set<set<int> > const &s)
    {
      cerr<<"{"<<endl;
      for(set<set<int> >::const_iterator i=s.begin();i!=s.end();i++)
	{
	  if(i!=s.begin())cerr<<endl<<",";
	  printSet(*i);
	}
      cerr<<"}"<<endl;
    }

    void flipNew(IntegerVector const &normal)
    {
      set<set<int> > toBeInserted;
      set<set<int> > toBeErased;
      int n=normal.size();
      set<int> nonZeroIndices;
      for(int i=0;i<n;i++)
	if(normal[i])nonZeroIndices.insert(i);

      for(set<int>::const_iterator i=nonZeroIndices.begin();i!=nonZeroIndices.end();i++)
	{
	  set<int> temp=nonZeroIndices;
	  temp.erase(*i);
	  if(normal[*i]>0)
	    toBeErased.insert(temp);
	  else
	    toBeInserted.insert(temp);
	}

      assert(toBeErased.size());

      /*      cerr<<"To be inserted"<<endl;
      printSetSet(toBeInserted);

      cerr<<"ToBeErased"<<endl;
      printSetSet(toBeErased);
      */

      set<int> A=*toBeErased.begin();
      set<set<int> > link;
      for(set<IntegerVector>::const_iterator i=bases.begin();i!=bases.end();i++)
	if(contains(*i,A))
	  link.insert(difference(*i,A));

      for(set<IntegerVector>::iterator i=bases.begin();i!=bases.end();)
	{
	  bool e=false;
	  for(set<set<int> >::const_iterator j=toBeErased.begin();j!=toBeErased.end();j++)
	    if(contains(*i,*j))
	      {
		e=true;
		break;
	      }
	  if(e)
	    {
	      set<IntegerVector>::iterator I=i;
	      I++;
	      bases.erase(i);
	      i=I;
	    }
	  else
	    i++;
	}

      for(set<set<int> >::iterator i=link.begin();i!=link.end();i++)
	{
	  for(set<set<int> >::iterator j=toBeInserted.begin();j!=toBeInserted.end();j++)
	    {
	      bases.insert(toVector(uni(*i,*j)));
	    }
	}
    }
    list<int> usedRays()const
    {
      list<int> ret;
      for(int i=0;i<A.getWidth();i++)
	{
	  bool contains=false;
	  for(set<IntegerVector>::const_iterator j=bases.begin();j!=bases.end();j++)
	    {
	      for(int k=0;k<j->size();k++)
		{
		  if((*j)[k]==i)
		    {
		      contains=true;
		      goto leave;
		    }
		}
	    }
	leave:
	  if(contains)ret.push_back(i);
	}
      return ret;
    }
    float hirschScore()const
    {
      int timesAttained;
      int nVertices=bases.size();
      int nEdges=coDimensionOneTriangles().size();
      int diameter=edgeGraph().diameter(&timesAttained);
      int dimension=A.getHeight();
      int nFacets=usedRays().size();
      
      if(diameter+dimension-nFacets>0)
	{
	  cerr<<"Counter example found\n";
	  //	  print();
	  assert(0);
	}
      
      return diameter+dimension-nFacets+(nFacets>120?(-0.01*nFacets):0);
    }
  };
  PolyhedralFan enumerate(Triangulation2 const &t)
  {
    PolyhedralFan ret(t.getN());
    list<Triangulation2> active;
    active.push_back(t);
    IntegerVectorList interiorPoints;
    interiorPoints.push_back(t.interiorPoint());
    while(!active.empty())
      {
	Triangulation2 a=active.front();
	PolyhedralCone C=a.secondaryCone();
	

	//	if(active.size()>100)break;//SLETMIGGGGG
	//log0 fprintf(stderr,"a\n");
	/*	{
	  PolyhedralCone C2=C;
	  C2.canonicalize();
	  }*/
	C.canonicalize();
	//log0 fprintf(stderr,"b\n");
	ret.insert(C);
	AsciiPrinter P(Stderr);
	//	C.print(&P);
	active.pop_front();
	//	fprintf(stderr,"pop\n");
	IntegerVectorList flips=a.facets();
	for(IntegerVectorList::const_iterator i=flips.begin();i!=flips.end();i++)
	  {
	    {
	      IntegerVectorList t=C.getEquations();
	      t.push_back(*i);
	      PolyhedralCone CF(C.getHalfSpaces(),t);
	      CF.findFacets();
	      //	      CF.canonicalize();
	    }

	    if(!i->isNonNegative()) //is this the right condition or should i be negated?
	    //   if(!(-*i).isNonNegative()) //is this the right condition or should i be negated?
	      {
		Triangulation2 b=a;
		log1 AsciiPrinter(Stderr)<<*i;
		/*fprintf(stderr,"Before:");
		  b.print(P);*/
		//		b.flip(*i);
		b.flipNew(-*i);
		/*fprintf(stderr,"After:");
		  b.print(P);*/
		if(!b.isEmpty())
		  {
		    IntegerVectorList inequalities=b.inequalities();
		    bool isKnown=false;
		    for(IntegerVectorList::const_iterator j=interiorPoints.begin();j!=interiorPoints.end();j++)
		      {
			bool match=true;
			for(IntegerVectorList::const_iterator k=inequalities.begin();k!=inequalities.end();k++)
			  {
			    if(dotLong(-*k,*j)<0)
			      {
				match=false;
				break;
			      }
			  }
			if(match)isKnown=true;
		      }
		    if(!isKnown)
		      {
			active.push_back(b);
			interiorPoints.push_back(b.interiorPoint());
		      }
		  }
	      }
	  }
      }
    return ret;
  }
  PolyhedralFan interactive(Triangulation2 const &t)
  {
    Triangulation2 a=t;

    while(1)
      {
	fprintf(stdout,"Triangles in current triangulation:%i\n",a.bases.size());
	PolyhedralCone C=a.secondaryCone();
	
	C.canonicalize();
	
	
	AsciiPrinter Pstd(Stderr);
	IntegerVectorList flips=a.facets();
	int I=0;
	for(IntegerVectorList::const_iterator i=flips.begin();i!=flips.end();i++,I++)
	  {
	    fprintf(stdout,"%i:\n",I);
	    Pstd.printVector(*i);
	    
	    
	    if(!i->isNonNegative())
	      {
		Triangulation2 b=a;
		//fprintf(stderr,"Before:");
		//b.print(P);
		//		b.flip(*i);
		/*		b.flipNew(*i);
		//fprintf(stderr,"After:");
		//b.print(P);
		fprintf(stdout,"Triangles in new triangulation:%i\n",b.bases.size());
		*/
	      }
	    
	    fprintf(stdout,"\n");
	  }
	int s;
	//cin >> s;
	fscanf(stdin,"%i",&s);
	if((s>=0)&&(s<I))
	  {
	    IntegerVectorList::const_iterator i=flips.begin();
	    for(int I=0;I<s;I++)i++;
	    a.flipNew(*i);
	  }
	else
	  {
	    a.print(Pstd);
	  }
      }
    PolyhedralFan ret(0);
    
    return ret;
  }
  PolyhedralFan automatic(Triangulation2 const &t, int abortAtSize)
  {
    Triangulation2 a=t;

    cout << "Looking for triangulation with "<<abortAtSize<<" simplices."<<endl;

    while(1)
      {
	fprintf(stdout,"Triangles in current triangulation:%i\n",a.bases.size());
	//	PolyhedralCone C=a.secondaryCone();
	
	//	C.canonicalize();
	
	
	AsciiPrinter Pstd(Stderr);
	IntegerVectorList flips=a.facets();
	int I=0;
	cerr << "Number of facets of secondary cone: " << flips.size() <<endl;
	for(IntegerVectorList::const_iterator i=flips.begin();i!=flips.end();i++,I++)
	  {
	    if(!i->isNonNegative())
	      {
		Triangulation2 b=a;
		b.flipNew(-*i);
		fprintf(stdout,"Triangles in new triangulation:%i\n",b.bases.size());

		if(b.bases.size()==abortAtSize)
		  {
		    b.print(Pstd);
		    
		    exit(0);
		  }

		if((b.bases.size()>a.bases.size())||((rand()&127)==0))
		  {
		    a=b;
		    break;
		  }
	      }
	  }
      }
    PolyhedralFan ret(0);
    
    return ret;
  }
  PolyhedralFan automaticHirsch(Triangulation2 const &t)
  {
    Triangulation2 a=t;
    while(1)
      {
	int nVertices=a.bases.size();
	int nEdges=a.coDimensionOneTriangles().size();
	int diameter=a.edgeGraph().diameter();
	int dimension=a.A.getHeight();
	int nFacets=a.usedRays().size();
	fprintf(stdout,"NVER: %i NEDGES: %i DIAMETER:%i DIMENSION:%i NFACETS:%i\n",nVertices,nEdges,diameter,dimension,nFacets);

	AsciiPrinter Pstd(Stderr);
	IntegerVectorList flips=a.facets();
	int I=0;
	float currentScore=a.hirschScore();
	cerr << "Current score: " << currentScore <<endl;
	for(IntegerVectorList::const_iterator i=flips.begin();i!=flips.end();i++,I++)
	  {
	    if(!i->isNonNegative())
	      {
		Triangulation2 b=a;
		b.flipNew(-*i);
		float bScore=b.hirschScore();
		fprintf(stdout,"New score:%f\n",bScore);

		if((bScore>currentScore)||((rand()&31)==0))
		  {
		    a=b;
		    break;
		  }
	      }
	  }
      }
    PolyhedralFan ret(0);
    
    return ret;
  }


  SimpleOption hirschOption;
  SimpleOption searchOption;
  IntegerOption scaleOption;
  SimpleOption symmetryOption;
public:
  const char *helpText()
  {
    return "This program computes the secondary fan of a vector configuration. The configuration is given as an ordered list of vectors. In order to compute the secondary fan of a point configuration an additional coordinate of ones must be added. For example {(1,0),(1,1),(1,2),(1,3)}.\n";
  }
  SecondaryFanApplication():
    searchOption("--unimodular","Use heuristics to search for unimodular triangulation rather than computing the complete secondary fan"),
    scaleOption("--scale","Assuming that the first coordinate of each vector is 1, this option will take the polytope in the 1 plane and scale it. The point configuration will be all lattice points in that scaled polytope. The polytope must have maximal dimension. When this option is used the vector configuration must have full rank. This option may be removed in the future."),
    symmetryOption("--symmetry","Tells the program to read in generators for a group of symmetries (subgroup of $S_n$) after having read in the vector configuration. The program checks that the configuration stays fixed when permuting the variables with respect to elements in the group. The output is grouped according to the symmetry.\n"),
    hirschOption("--hirsch","")
  {
    hirschOption.hide();
    registerOptions();
  }    

  char *name()
  {
    return "_secondaryfan";
  }

  int main()
  {
    IntegerMatrix A=rowsToIntegerMatrix(FileParser(Stdin).parseIntegerVectorList()).transposed();

    int n=A.getWidth();

    SymmetryGroup s(n);
    if(symmetryOption.getValue())
      {
	IntegerVectorList generators=FileParser(Stdin).parseIntegerVectorList();
	for(IntegerVectorList::const_iterator i=generators.begin();i!=generators.end();i++)
	  {
	    assert(i->size()==n);
	    FieldMatrix M1=integerMatrixToFieldMatrix(A,Q);
	    FieldMatrix M2=integerMatrixToFieldMatrix(rowsToIntegerMatrix(SymmetryGroup::permuteIntegerVectorList(A.getRows(),*i)),Q);
	    M1.reduce();
	    M1.REformToRREform(true);
	    M2.reduce();
	    M2.REformToRREform(true);
	    
	    if(!(M1==M2))
	      {
		AsciiPrinter(Stderr) << "Permutation "<< *i <<
		  " does not keep the configuration fixed.\n";
		assert(0);
	      }
	  }
	s.computeClosure(generators);
      }


    if(scaleOption.getValue())
      {
	if(rank(A)!=A.getHeight())
	  {
	    cerr << "The vector configuration must have full rank in order to use the scale option.\n";
	    assert(0);
	  }

	int s=scaleOption.getValue();
	
	cout << "Input configuration:" << endl;
	AsciiPrinter(Stdout)<<A.transposed().getRows();
	
	IntegerVectorList empty;
	PolyhedralCone dual(A.transposed().getRows(),empty,n);
	dual.canonicalize();
	assert(dual.dimensionOfLinealitySpace()==0);
	IntegerVectorList inequalities=dual.extremeRays();
	
	IntegerMatrix M1=rowsToIntegerMatrix(inequalities);
	IntegerMatrix M2=-1*M1.submatrix(0,1,M1.getHeight(),M1.getWidth());
	IntegerVector rightHandSide=s*M1.transposed().submatrix(0,0,1,M1.getWidth())[0];
	IntegerVector v=s*A.submatrix(1,0,A.getHeight(),1).transposed()[0];
	
	IntegerVectorList l=intsInPolytopeGivenIneqAndPt(M2,rightHandSide,v);

	IntegerVectorList lT=rowsToIntegerMatrix(l).transposed().getRows();
	lT.push_front(IntegerVector::allOnes(l.size()));
	l=rowsToIntegerMatrix(lT).transposed().getRows();

	cout << "New configuration:" << endl;
	AsciiPrinter(Stdout)<<l;
	A=rowsToIntegerMatrix(l).transposed();
      }
    
    /* If the vector configuration A does not have full rank then
       change coordinates. */
    if(rank(A)!=A.getHeight())
      {
	FieldMatrix M=integerMatrixToFieldMatrix(A,Q);
	M.reduce(false,true);//force integer operations - preserving volume
	M.removeZeroRows();
	A=fieldMatrixToIntegerMatrix(M);
      }


    Triangulation2 t;
    t.A=A;

    /* Convert a Triangulation to a Triangulation2 */
    {
      list<Triangulation::Cone> T=Triangulation::triangulate(A.transposed());
      for(list<Triangulation::Cone>::const_iterator i=T.begin();i!=T.end();i++)
	{
	  IntegerVector v=i->size();
	  int J=0;
	  for(Triangulation::Cone::const_iterator j=i->begin();j!=i->end();j++,J++)
	    v[J]=*j;
	    t.bases.insert(v);
	}
    }
    
    if(searchOption.getValue())
      {
	PolyhedralFan f=automatic(t,t.totalVolume());
      }
    else if(hirschOption.getValue())
      {
	PolyhedralFan f=automaticHirsch(t);
      }
    else
      {
	PolyhedralFan f=enumerate(t).negated();//Changing sign

	//        f.printWithIndices(&p,false,0,false);
	AsciiPrinter p(Stdout);
        f.printWithIndices(&p,
			   FPF_default|
			   (symmetryOption.getValue()?FPF_group|FPF_conesCompressed:0),
			   &s);
        //f.printWithIndices(&p,false,&s,symmetryOption.getValue());
      }
    return 0;
  }
};

static SecondaryFanApplication theApplication;

