#include "symmetry.h"

#include "printer.h"
#include <iostream>
IntegerVector SymmetryGroup::identity(int n)
{
  IntegerVector v(n);
  for(int i=0;i<n;i++)v[i]=i;

  return v;
}


IntegerVector SymmetryGroup::inverse(IntegerVector const &a)
{
  return composeInverse(a,identity(a.size()));
}


SymmetryGroup::SymmetryGroup(int n):
  byteTable(0)
{
  elements.insert(identity(n));
}


int SymmetryGroup::sizeOfBaseSet()const
{
  assert(!elements.empty());
  return elements.begin()->size();
}

void SymmetryGroup::computeClosure(IntegerVector const &v) //does this work??
{  
  ElementContainer newOnes;

  newOnes.insert(v);

  while(!newOnes.empty())
    {
      static int i;
      i++;
      if((i&127)==0)fprintf(Stderr,"%i\n",i);


      IntegerVector v=*newOnes.begin();
      for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++)
	{
	  {
	    IntegerVector n(compose(*i,v));
	    if(0==elements.count(n))
	      newOnes.insert(n);
	  }
	  {
	    IntegerVector n(compose(v,*i));
	    if(0==elements.count(n))
	      newOnes.insert(n);
	  }
	}
      newOnes.erase(v);
      elements.insert(v);
    }
}


void SymmetryGroup::computeClosure(IntegerVectorList const &l)
{
  //  for(IntegerVectorList::const_iterator i=l.begin();i!=l.end();i++)
  //  computeClosure(*i);

  bool growing=true;
  while(growing)
    {
      growing=false;
      for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++)
	{
	  for(IntegerVectorList::const_iterator j=l.begin();j!=l.end();j++)
	    {
	      {
		IntegerVector n(compose(*i,*j));
		growing|=(0==elements.count(n));
		elements.insert(n);
	      }
	      {
		IntegerVector n(compose(*i,*j));
		growing|=(0==elements.count(n));
		elements.insert(n);
	      }
	    }
	}
    }
}


void SymmetryGroup::print(FILE *f)
{
  AsciiPrinter P(f);
  P.printString("Printing SymmetryGroup\n");
  IntegerVectorList l;
  for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++)
    {
      //      P.printVector(*i);
      //      P.printNewLine();
      l.push_back(*i);
    }
  P.printVectorList(l);
  fprintf(f,"Group order=%i\n",elements.size());
  P.printString("Done printing SymmetryGroup.\n");
}


IntegerVector SymmetryGroup::compose(IntegerVector const &perm, IntegerVector const &b)
{
  IntegerVector v(perm);
  assert(perm.size()==b.size());
  for(int i=0;i<perm.size();i++)v[i]=b[perm[i]];
  return v;
}


IntegerVector SymmetryGroup::composeInverse(IntegerVector const &a, IntegerVector const &b)
{
  IntegerVector v(a);
  assert(a.size()==b.size());
  for(int i=0;i<a.size();i++)v[a[i]]=b[i];
  return v;
}


IntegerVectorList SymmetryGroup::permuteIntegerVectorList(IntegerVectorList const &l, IntegerVector const &v)
{
  IntegerVectorList ret;
  for(IntegerVectorList::const_iterator i=l.begin();i!=l.end();i++)
    ret.push_back(compose(v,*i));

  return ret;
}


Polynomial SymmetryGroup::permutePolynomial(Polynomial const &p, IntegerVector const &v)
{
  Polynomial q(p.getRing());

  for(TermMap::const_iterator i=p.terms.begin();i!=p.terms.end();i++)
    {
      q+=Term(i->second,Monomial(p.getRing(),compose(v,i->first.exponent)));
    }

  q.mark(Monomial(p.getRing(),compose(v,p.getMarked().m.exponent)));

  return q;
}


PolynomialSet SymmetryGroup::permutePolynomialSet(PolynomialSet const &s, IntegerVector const &v)
{
  PolynomialRing theRing=s.getRing();
  PolynomialSet ret(theRing);
  for(PolynomialSet::const_iterator i=s.begin();i!=s.end();i++)
    {
      ret.push_back(permutePolynomial(*i,v));
    }

  return ret;
}


Polynomial SymmetryGroup::computeUniqueRepresentative(Polynomial p)
{
  Polynomial best=p;

  for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++)
    {
      Polynomial q=permutePolynomial(p,*i);
      if(PolynomialCompare()(best,q))best=q;
    }
  return best;
}


IntegerVector SymmetryGroup::orbitRepresentative(IntegerVector const &v, IntegerVector *usedPermutation)const
{
  IntegerVector ret=v;
  ElementContainer::const_iterator usedPerm;
  for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++)
    {
      IntegerVector q=compose(*i,v);
      if(! (q<ret))//negation to make sure that usedPerm is set
	{
	  usedPerm=i;
	  ret=q;
	}
    }

  if(usedPermutation)*usedPermutation=*usedPerm;
  return ret;
}

IntegerVector SymmetryGroup::orbitRepresentativeFixing(IntegerVector const &v, IntegerVector const &fixed)const
{
  IntegerVector ret=v;

  for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++)
    if(compose(*i,fixed)==fixed)
      {
	IntegerVector q=compose(*i,v);
	if(ret<q)ret=q;
      }
  return ret;
}

bool SymmetryGroup::isPermutation(IntegerVector const &a)
{
  int n=a.size();
  IntegerVector temp(n);
  for(int i=0;i<n;i++)temp[i]=-1;
  for(int i=0;i<n;i++)
    {
      if(a[i]<0 || a[i]>=n)return false;
      temp[i]=i;
    }
  for(int i=0;i<n;i++)if(temp[i]<0)return false;
  return true;
}


int SymmetryGroup::orbitSize(IntegerVector const &stable)const
{
  int groupSize=elements.size();

  int n=stable.size();
  int numFixed=0;
  for(SymmetryGroup::ElementContainer::const_iterator j=elements.begin();j!=elements.end();j++)
    {
      bool doesFix=true;

      for(int i=0;i<n;i++)
	if(stable[i]!=stable[(*j)[i]])
	  {
	    doesFix=false;
	    break;
	  }
      if(doesFix)numFixed++;
    }
  return groupSize/numFixed;
}


IntegerVector SymmetryGroup::fundamentalDomainInequality(IntegerVector const &perm)
{
  for(int i=0;i<perm.size();i++)
    if(perm[i]!=i)
      return IntegerVector::standardVector(perm.size(),i)-IntegerVector::standardVector(perm.size(),perm[i]);
      //      return -IntegerVector::standardVector(perm.size(),i)-IntegerVector::standardVector(perm.size(),perm[i]);//USE this for 4x4 of 5x5
  return IntegerVector(perm.size());
}


IntegerVectorList SymmetryGroup::fundamentalDomainInequalities()const
{
  set<IntegerVector> ret2;
  for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++)
    ret2.insert(fundamentalDomainInequality(*i));

  IntegerVectorList ret;
  for(set<IntegerVector>::const_iterator i=ret2.begin();i!=ret2.end();i++)
    if(!i->isZero())ret.push_back(*i);

  return ret;
}


void SymmetryGroup::createByteTable()
{
  assert(!byteTable);
  int n=sizeOfBaseSet();
  byteTableHeight=elements.size();
  byteTable=(unsigned char*)malloc(n*byteTableHeight*sizeof(unsigned char));
  assert(byteTable);
  int j=0;
  for(ElementContainer::const_iterator i=elements.begin();i!=elements.end();i++,j++)
    {
      for(int k=0;k<n;k++)
	byteTable[j*n+k]=(*i)[k];
    }
}


unsigned char * SymmetryGroup::getByteTable()const
{
  return byteTable;
}


int  SymmetryGroup::getByteTableHeight()const
{
  return byteTableHeight;
}
